#!/bin/sh

# =============================================================================

: << END-OF-MANUAL-PAGE
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1991 OTC LIMITED
//     Copyright 1994-1998 DUMPLETON SOFTWARE CONSULTING PTY LIMITED

// [info2html]
//
// = TITLE
//     Generates HTML manual pages from a classinfo file.
//
// = SYNOPSIS
//     info2html [ -ext html ] file.ci
//     info2html [ -nested ] file.ci
//     info2html [ -link 'path/*.html' ] file.ci
//
// = DESCRIPTION
//     The <info2html> program generates a HTML style manual page
//     from the classinfo file specified. The names of the generated files
//     will correspond to the names of the classes the classinfo file
//     describes.
//
//     By default, manual pages for nested classes will not be output.
//     To generated manual pages for nested classes specify the "-nested"
//     option.
//
//     By default, the suffix appended to the manual pages will be ".html".
//     The extension appended to the manual pages can be changed by specifying
//     the "-ext" option.
//
//     By default, no cross references are inserted into generated file.
//     Cross references can be generated by supplying "-link" with the
//     argument being a glob style pattern of the files which can be linked
//     to. Note that the files must exist, so this program might need to
//     be run twice, the first time to generate the files, the second
//     time so that links can be put in.
//
// = EXAMPLE
// = BEGIN<CODE>
//     class2info foo.hh
//     info2html foo.ci
// = END<CODE>
//
// = NOTES
//     Although reference to generating manual pages for only classes is
//     mentioned, manual pages for structs, unions and templates are
//     also generated. Further, if the syntax for standalone manual
//     pages has been used, then manual pages will be generated for that
//     also
//
// = SEE ALSO
//     class2info(1), class2html(1)
END-OF-MANUAL-PAGE

# =============================================================================

OSE_HOST=@OSE_HOST@

OSE_RELEASE_DIRECTORY=@OSE_RELEASE_DIRECTORY@
export OSE_RELEASE_DIRECTORY

OSE_ROOT=${OSE_ROOT-@OSE_ROOT@}
export OSE_ROOT

OSE_VERSION_ROOT=$OSE_ROOT/$OSE_RELEASE_DIRECTORY
export OSE_VERSION_ROOT

BINDIR="$OSE_VERSION_ROOT/$OSE_HOST/bin"
LIBDIR=${CLASSINFOLIBDIR-"$OSE_VERSION_ROOT/lib"}

AWK="${AWK-@AWK@}"

if test "$AWK" = "gawk"
then
  VARG="-v"
fi

INFO2MAN=$LIBDIR/info2doc.awk
MANFMT=${CLASSINFOMANFMT-"$LIBDIR/info2doc.fmt"}

FILES=
PATTERNS=
MANEXT="html"
NESTED=

#
# Error.
#
ERROR()
{
  echo "`basename $0`: $1" >&2
  shift
  while test $# != "0"
  do
    echo $1 >&2
    shift
  done
  exit 1
}

#
# Usage message.
#
USAGE()
{
  ERROR "Usage: `basename $0` file.ci"
}

#
# Parse command line.
#
while test $# != 0
do
  case $1 in
    -link)
      shift
      if test "$#" != "0"
      then
	PATTERNS=$1
	shift
      fi
      ;;
    -ext)
      shift
      if test "$#" != "0"
      then
	MANEXT=$1
	shift
      fi
      ;;
    -fmt)
      shift
      if test "$#" != "0"
      then
	MANFMT=$1
	shift
      fi
      ;;
    -nested)
      shift
      NESTED=1
      ;;
    *.ci)
      FILES="$FILES $1"
      shift
      ;;
    *)
      USAGE
      ;;
    esac
done

#
# Check usage.
#
if test "$FILES" = ""
then
  USAGE
fi

#
# Check for awk file etc.
#

if test ! -f $INFO2MAN
then
  ERROR "Can't find $INFO2MAN"
fi

if test ! -f $MANFMT
then
  ERROR "Can't find $MANFMT"
fi

for input in $FILES
do
  ADTS=`$AWK $VARG nested="$NESTED" '
  BEGIN {
    FS="\n"; RS=""
  }
  $1 ~ "^(CLASS|STRUCT|UNION|MANUAL)$" && $2 !~ "(::|<)" {
    printf( "%s\n", $2 )
  }
  $1 ~ "^(CLASS|STRUCT|UNION)$" && $2 ~ "::" && $2 !~ "(<)" {
    if ( nested == "1" && $2 !~ "::ANON" )
      printf( "%s\n", $2 )
  }
  $1 ~ "^TEMPLATE$" && $3 !~ "::" {
    printf( "%s\n", $3 )
  }
  $1 ~ "^TEMPLATE$" && $3 ~ "::" {
    if ( nested == "1" && $3 !~ "::ANON" )
      printf( "%s\n", $3 )
  }' $input`

  if test ! -z "$ADTS"
  then
    for ADT in $ADTS
    do
      echo "$ADT"

      $AWK -f $INFO2MAN $VARG pass=0 $VARG device=html $VARG infile=$input \
       $VARG class=$ADT $VARG manext="$MANEXT" $MANFMT $MANFMT | \
       $AWK -f ${LIBDIR}/man2html2.awk $VARG link="$PATTERNS" > \
       `echo $ADT | tr : _`.$MANEXT
    done
  fi
done
