import netsvc
import signal
import StringIO
import string
import cgi

class FormServlet(netsvc.HttpServlet):

  def __init__(self,session,serverRoot,servletPath):
    netsvc.HttpServlet.__init__(self,session)
    self._serverRoot = serverRoot
    self._servletPath = servletPath
    self._content = []
    self._contentLength = 0

  def serverRoot(self):
    return self._serverRoot

  def servletPath(self):
    return self._servletPath

  def processRequest(self):
    if self.requestMethod() not in ["GET","POST"]:
      self.sendError(501,"Request method type is not supported.")
    if self.requestMethod() == "POST" and self.contentLength() < 0:
      self.sendError(400,"Content length required for POST.")
    if self.requestMethod() == "GET" and self.contentLength() > 0:
      self.sendError(400,"Unexpected content length for GET.")
    if self.requestMethod() == "GET":
      environ = {}
      environ["REQUEST_METHOD"] = self.requestMethod()
      environ["QUERY_STRING"] = self.queryString()
      headers = self.headers()
      headers["content-type"] = "application/x-www-form-urlencoded"
      try:
	form = cgi.FieldStorage(headers=headers,environ=environ,
	    keep_blank_values=1)
	self.processForm(form)
      except:
        netsvc.logException()
        self.shutdown()
    elif self.requestMethod() == "POST" and self.contentLength() == 0:
      environ = {}
      environ["REQUEST_METHOD"] = self.requestMethod()
      fp = StringIO.StringIO("")
      headers = self.headers()
      try:
	form = cgi.FieldStorage(headers=headers,environ=environ,
	    keep_blank_values=1,fp=fp)
	self.processForm(form)
      except:
        netsvc.logException()
        self.shutdown()

  def processContent(self,content):
    self._content.append(content)
    self._contentLength = self._contentLength + len(content)
    if self._contentLength >= self.contentLength():
      environ = {}
      environ["REQUEST_METHOD"] = self.requestMethod()
      self._content = string.join(self._content,"")
      self._content = self._content[:self.contentLength()]
      fp = StringIO.StringIO(self._content)
      headers = self.headers()
      try:
	form = cgi.FieldStorage(headers=headers,environ=environ,
	    keep_blank_values=1,fp=fp)
	self.processForm(form)
      except:
        netsvc.logException()
        self.shutdown()

  def processForm(self,form):
    self.sendResponse(501)


class TestServlet(FormServlet):

  def __init__(self,session,serverRoot,servletPath):
    FormServlet.__init__(self,session,serverRoot,servletPath)
    self._batch = None
    self._total = None
    self._count = 0
    self._job = netsvc.Job(self.generateContent)

  def destroyServlet(self):
    FormServlet.destroyServlet(self)
    self._job.cancel()
    self._job = None

  def processForm(self,form):
    if not form:
      self.sendResponse(200)
      self.sendHeader("Content-Type","text/html")
      self.endHeaders()
      values = { "servlet": self.servletPath() }
      self.sendContent(FORM%values)
      self.endContent()
    else:
      try:
        self._batch = int(form["batch"].value)
        self._total = int(form["total"].value)
      except:
	self.sendResponse(200)
	self.sendHeader("Content-Type","text/html")
	self.endHeaders()
	values = { "servlet": self.servletPath() }
	self.sendContent(FORM%values)
	self.endContent()
      else:
	self.sendResponse(200)
	self.sendHeader("Content-Type","text/plain")
	self.endHeaders()
	self.monitorCongestion(self.clientCongestion)
        self._job.schedule(netsvc.IDLE_JOB)

  def generateContent(self):
    content = []
    for i in range(0,self._batch):
      self._count = self._count + 1
      content.append(string.zfill(self._count,60))
    content.append("")
    self.sendContent(string.join(content,"\n"))
    self._total = self._total - 1
    if self._total <= 0:
      self.ignoreCongestion()
      self.endContent()
    else:
      self._job.schedule(netsvc.IDLE_JOB)

  def clientCongestion(self,status,pending,threshold):
    if status == netsvc.CONNECTION_CLEARED:
      self._job.reset()
      self._job.schedule(netsvc.IDLE_JOB)
    elif status == netsvc.CONNECTION_BLOCKED:
      self._job.cancel()


class HttpServer(netsvc.HttpServer):

  def __init__(self):
    netsvc.HttpServer.__init__(self)

  def servlet(self,serverRoot,servletPath,session):
    if servletPath == "":
      return TestServlet(session,serverRoot,servletPath)
    return None


FORM = """
<html>
<body>
<form method="POST" action="%(servlet)s">
<pre>
Batch: <input type="text" name="batch" value="10"></br>
Total: <input type="text" name="total" value="10"></br>
<input type="submit" name="submit" value="Submit">
</pre>
</form>
</body>
</html>
"""


dispatcher = netsvc.Dispatcher()
dispatcher.monitor(signal.SIGINT)

daemon = netsvc.HttpDaemon(8000)
server = HttpServer()
daemon.attach("/",server)
daemon.start()

dispatcher.run()
