#ifndef OTC_COLLCTN_TDUALMAP_C
#define OTC_COLLCTN_TDUALMAP_C
/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     collctn/tdualmap.c
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1996-1997 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#include <OTC/collctn/linklist.hh>
#include <OTC/collctn/kcursor.hh>
#include <OTC/collctn/icursor.hh>
#include <OTC/collctn/kicursor.hh>
#include <OTC/collctn/prbucket.hh>

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_TDualMap<T1,T2>::~OTC_TDualMap()
{
  count_->unReference();
  itemIndex_->unReference();
  keyIndex_->unReference();
  listByItems_->unReference();
  listByKeys_->unReference();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_TDualMap<T1,T2>::OTC_TDualMap(
 int (*theKeyRankFn)(T1 const&,T1 const&),
 int (*theItemRankFn)(T2 const&,T2 const&)
)
  : keyRankFn_(theKeyRankFn), itemRankFn_(theItemRankFn)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
#endif

#if defined(ENV_OSTORE)
  keyIndex_ = new (theLocality,OTC_AVLTree::get_os_typespec()) OTC_AVLTree;
#else
  keyIndex_ = new OTC_AVLTree;
#endif
  OTCLIB_ASSERT_M(keyIndex_ != 0);
  keyIndex_->reference();

#if defined(ENV_OSTORE)
  itemIndex_ = new (theLocality,OTC_AVLTree::get_os_typespec()) OTC_AVLTree;
#else
  itemIndex_ = new OTC_AVLTree;
#endif
  OTCLIB_ASSERT_M(itemIndex_ != 0);
  itemIndex_->reference();

#if defined(ENV_OSTORE)
  listByKeys_ = new (theLocality,OTC_LinkList::get_os_typespec()) OTC_LinkList;
#else
  listByKeys_ = new OTC_LinkList;
#endif
  OTCLIB_ASSERT_M(listByKeys_ != 0);
  listByKeys_->reference();

#if defined(ENV_OSTORE)
  listByItems_ = new (theLocality,OTC_LinkList::get_os_typespec()) OTC_LinkList;
#else
  listByItems_ = new OTC_LinkList;
#endif
  OTCLIB_ASSERT_M(listByItems_ != 0);
  listByItems_->reference();

#if defined(ENV_OSTORE)
  count_ = new (theLocality,OTC_RCount::get_os_typespec()) OTC_RCount;
#else
  count_ = new OTC_RCount;
#endif
  OTCLIB_ASSERT_M(count_ != 0);
  count_->reference();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_TDualMap<T1,T2>::OTC_TDualMap(OTC_TDualMap<T1,T2> const& theMap)
  : keyRankFn_(theMap.keyRankFn_), itemRankFn_(theMap.itemRankFn_)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
#endif

#if defined(ENV_OSTORE)
  keyIndex_ = new (theLocality,OTC_AVLTree::get_os_typespec()) OTC_AVLTree;
#else
  keyIndex_ = new OTC_AVLTree;
#endif
  OTCLIB_ASSERT_M(keyIndex_ != 0);
  keyIndex_->reference();

#if defined(ENV_OSTORE)
  itemIndex_ = new (theLocality,OTC_AVLTree::get_os_typespec()) OTC_AVLTree;
#else
  itemIndex_ = new OTC_AVLTree;
#endif
  OTCLIB_ASSERT_M(itemIndex_ != 0);
  itemIndex_->reference();

#if defined(ENV_OSTORE)
  listByKeys_ = new (theLocality,OTC_LinkList::get_os_typespec()) OTC_LinkList;
#else
  listByKeys_ = new OTC_LinkList;
#endif
  OTCLIB_ASSERT_M(listByKeys_ != 0);
  listByKeys_->reference();

#if defined(ENV_OSTORE)
  listByItems_ = new (theLocality,OTC_LinkList::get_os_typespec())
OTC_LinkList;
#else
  listByItems_ = new OTC_LinkList;
#endif
  OTCLIB_ASSERT_M(listByItems_ != 0);
  listByItems_->reference();

#if defined(ENV_OSTORE)
  count_ = new (theLocality,OTC_RCount::get_os_typespec()) OTC_RCount;
#else
  count_ = new OTC_RCount;
#endif
  OTCLIB_ASSERT_M(count_ != 0);
  count_->reference();

  OTC_PairIterator<T1,T2> thePairs;
  thePairs = theMap.pairs();
  while (thePairs.isValid())
  {
    add(thePairs.key(),thePairs.item());
    thePairs.next();
  }
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_TDualMap<T1,T2>::OTC_TDualMap(
 OTC_TDualMap<T1,T2> const& theMap,
 OTC_ShallowCopy
)
  : keyRankFn_(theMap.keyRankFn_), itemRankFn_(theMap.itemRankFn_)
{
  keyIndex_ = theMap.keyIndex_;
  keyIndex_->reference();

  itemIndex_ = theMap.itemIndex_;
  itemIndex_->reference();

  listByKeys_ = theMap.listByKeys_;
  listByKeys_->reference();

  listByItems_ = theMap.listByItems_;
  listByItems_->reference();

  count_ = theMap.count_;
  count_->reference();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
void OTC_TDualMap<T1,T2>::removeAll()
{
  count_->set(0);
  itemIndex_->removeAll();
  keyIndex_->removeAll();
  listByKeys_->removeAll();
  listByItems_->removeAll();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
u_int OTC_TDualMap<T1,T2>::frequencyKey(T1 const& theKey) const
{
  OTC_AVLTreeNode* theIndexNode;
  theIndexNode = (OTC_AVLTreeNode*)keyIndex_->root();

  if (searchIndexKey_(theIndexNode,theKey,theIndexNode) == 0)
    return theIndexNode->index()->population();

  return 0;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
u_int OTC_TDualMap<T1,T2>::frequencyItem(T2 const& theItem) const
{
  OTC_AVLTreeNode* theIndexNode;
  theIndexNode = (OTC_AVLTreeNode*)itemIndex_->root();

  if (searchIndexItem_(theIndexNode,theItem,theIndexNode) == 0)
    return theIndexNode->index()->population();

  return 0;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_Boolean OTC_TDualMap<T1,T2>::containsKey(T1 const& theKey) const
{
  OTC_AVLTreeNode* theIndexNode;
  theIndexNode = (OTC_AVLTreeNode*)keyIndex_->root();

  return searchIndexKey_(theIndexNode,theKey,theIndexNode) == 0;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_Boolean OTC_TDualMap<T1,T2>::containsItem(T2 const& theItem) const
{
  OTC_AVLTreeNode* theIndexNode;
  theIndexNode = (OTC_AVLTreeNode*)itemIndex_->root();

  return searchIndexItem_(theIndexNode,theItem,theIndexNode) == 0;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_Boolean OTC_TDualMap<T1,T2>::contains(
 T1 const& theKey,
 T2 const& theItem
) const
{
  OTC_AVLTreeNode* theIndexNode;
  theIndexNode = (OTC_AVLTreeNode*)keyIndex_->root();

  if (searchIndexKey_(theIndexNode,theKey,theIndexNode) != 0)
    return OTCLIB_FALSE;

  OTC_AVLLinkNode* theLinkNode;
  theLinkNode = (OTC_AVLLinkNode*)theIndexNode->index()->root();

  return searchGroupItem_(theLinkNode,theItem,theLinkNode) == 0;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_TDualMap<T1,T2>& OTC_TDualMap<T1,T2>::operator=(
 OTC_TDualMap<T1,T2> const& theMap
)
{
  if (&theMap != this)
  {
    removeAll();

    OTC_PairIterator<T1,T2> thePairs;
    thePairs = theMap.pairs();
    while (thePairs.isValid())
    {
      add(thePairs.key(),thePairs.item());
      thePairs.next();
    }
  }

  return *this;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
void OTC_TDualMap<T1,T2>::add(T1 const& theKey, T2 const& theItem)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
#endif

  int theRank;
  OTC_Link* theLink;
  OTC_AVLLinkNode* theLinkNode;
  OTC_AVLLinkNode* theNewLinkNode;
  OTC_AVLTreeNode* theIndexNode;

  // Search for key.

  theIndexNode = (OTC_AVLTreeNode*)keyIndex_->root();

  theRank = searchIndexKey_(theIndexNode,theKey,theIndexNode);

  if (theRank != 0)
  {
    // Key does not exist.

    OTC_AVLTreeNode* theNewIndexNode;
#if defined(ENV_OSTORE)
    theNewIndexNode = new (theLocality,OTC_AVLTreeNode::get_os_typespec())
     OTC_AVLTreeNode;
#else
    theNewIndexNode = new OTC_AVLTreeNode;
#endif
    OTCLIB_ASSERT_M(theNewIndexNode != 0);

    if (theIndexNode == 0)
    {
      keyIndex_->addRoot(theNewIndexNode);
    }
    else if (theRank < 0)
    {
      theIndexNode->addBefore(theNewIndexNode);
    }
    else
    {
      theIndexNode->addAfter(theNewIndexNode);
    }

    theIndexNode = theNewIndexNode;
  }

  // Search for item.

  theLinkNode = (OTC_AVLLinkNode*)theIndexNode->index()->root();

  theRank = searchGroupItem_(theLinkNode,theItem,theLinkNode);

  OTCLIB_ENSURE_FN((theRank != 0),
   "OTC_TDualMap<T1,T2>::add(T1 const&, T2 const&)",
   "key/item pair exists");

  // Item does not exist.

  theLink = link_(theKey,theItem);

#if defined(ENV_OSTORE)
  theNewLinkNode = new (theLocality,OTC_AVLLinkNode::get_os_typespec())
   OTC_AVLLinkNode(theLink);
#else
  theNewLinkNode = new OTC_AVLLinkNode(theLink);
#endif
  OTCLIB_ASSERT_M(theNewLinkNode != 0);

  if (theLinkNode == 0)
  {
    theIndexNode->index()->addRoot(theNewLinkNode);
    listByKeys_->addLast(theLink);
  }
  else if (theRank < 0)
  {
    theLinkNode->addBefore(theNewLinkNode);
    theLinkNode->link()->addBefore(theLink);
  }
  else
  {
    theLinkNode->addAfter(theNewLinkNode);
    theLinkNode->link()->addAfter(theLink);
  }

  // Now do it all again for the item map.

  theIndexNode = (OTC_AVLTreeNode*)itemIndex_->root();

  theRank = searchIndexItem_(theIndexNode,theItem,theIndexNode);

  if (theRank != 0)
  {
    // Item does not exist.

    OTC_AVLTreeNode* theNewIndexNode;
#if defined(ENV_OSTORE)
    theNewIndexNode = new (theLocality,OTC_AVLTreeNode::get_os_typespec())
     OTC_AVLTreeNode;
#else
    theNewIndexNode = new OTC_AVLTreeNode;
#endif
    OTCLIB_ASSERT_M(theNewIndexNode != 0);

    if (theIndexNode == 0)
    {
      itemIndex_->addRoot(theNewIndexNode);
    }
    else if (theRank < 0)
    {
      theIndexNode->addBefore(theNewIndexNode);
    }
    else
    {
      theIndexNode->addAfter(theNewIndexNode);
    }

    theIndexNode = theNewIndexNode;
  }

  // Search for item.

  theLinkNode = (OTC_AVLLinkNode*)theIndexNode->index()->root();

  theRank = searchGroupKey_(theLinkNode,theKey,theLinkNode);

  OTCLIB_ENSURE_FN((theRank != 0),
   "OTC_TDualMap<T1,T2>::add(T1 const&, T2 const&)",
   "key/item pair exists");

  // Key does not exist.

  theLink = link_(theKey,theItem);

#if defined(ENV_OSTORE)
  theNewLinkNode = new (theLocality,OTC_AVLLinkNode::get_os_typespec())
   OTC_AVLLinkNode(theLink);
#else
  theNewLinkNode = new OTC_AVLLinkNode(theLink);
#endif
  OTCLIB_ASSERT_M(theNewLinkNode != 0);

  if (theLinkNode == 0)
  {
    theIndexNode->index()->addRoot(theNewLinkNode);
    listByItems_->addLast(theLink);
  }
  else if (theRank < 0)
  {
    theLinkNode->addBefore(theNewLinkNode);
    theLinkNode->link()->addBefore(theLink);
  }
  else
  {
    theLinkNode->addAfter(theNewLinkNode);
    theLinkNode->link()->addAfter(theLink);
  }

  count_->inc();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_PairIterator<T1,T2> OTC_TDualMap<T1,T2>::pairs(
 OTC_Direction theDirection,
 OTC_Protection theProtection
) const
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
  if (!theLocality.isTransientSegment())
  {
    os_transaction* theTXN = os_transaction::get_current();
    if (theTXN != 0 && theTXN->get_type() == os_transaction::read_only)
      theProtection = OTCLIB_UNSAFE;
 
    if (theProtection == OTCLIB_UNSAFE)
      theLocality = os_segment::get_transient_segment();
  }
#endif
 
  OTC_PairCursor<T1,T2>* theIter;
#if defined(ENV_OSTORE)
  theIter = new (theLocality,
   OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >::get_os_typespec())
   OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >(listByKeys_,theDirection,
   theProtection);
#else
  theIter = new OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >(
   listByKeys_,theDirection,theProtection);
#endif
  OTCLIB_ASSERT_M(theIter != 0);
  return theIter;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_Cursor<T1>* OTC_TDualMap<T1,T2>::keys_(
 T2 const& theItem,
 int (*theRankFn)(T2 const&, T2 const&),
 OTC_Direction theDirection,
 OTC_Protection theProtection
) const
{
  OTC_AVLTreeNode* theNode;
  OTC_AVLTreeNode* theStartNode;
  OTC_AVLTreeNode* theEndNode;

  theNode = (OTC_AVLTreeNode*)itemIndex_->root();

  theNode = rangeIndexItem_(theNode,theItem,theRankFn,theStartNode,theEndNode);

  if (theNode == 0)
    return 0;

  OTC_Linkable* theStartLink;
  OTC_Linkable* theEndLink;
  OTC_AVLLinkNode* theStartLinkNode;
  OTC_AVLLinkNode* theEndLinkNode;

  theStartLinkNode = (OTC_AVLLinkNode*)theStartNode->index()->first();
  theEndLinkNode = (OTC_AVLLinkNode*)theEndNode->index()->last();

  theStartLink = theStartLinkNode->link()->prev();
  theEndLink = theEndLinkNode->link()->next();

#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
  if (!theLocality.isTransientSegment())
  {
    os_transaction* theTXN = os_transaction::get_current();
    if (theTXN != 0 && theTXN->get_type() == os_transaction::read_only)
      theProtection = OTCLIB_UNSAFE;

    if (theProtection == OTCLIB_UNSAFE)
      theLocality = os_segment::get_transient_segment();
  }
#endif

  OTC_Anchor* theStartAnchor = 0;
  OTC_Anchor* theEndAnchor = 0;

  if (theProtection == OTCLIB_SAFE)
  {
#if defined(ENV_OSTORE)
    theStartAnchor = new (theLocality,OTC_Anchor::get_os_typespec())
OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new (theLocality,OTC_Anchor::get_os_typespec()) OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#else
    theStartAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#endif
    theStartLink->addAfter(theStartAnchor);
    theStartLink = theStartAnchor;
    theEndLink->addBefore(theEndAnchor);
    theEndLink = theEndAnchor;
  }

  OTC_Cursor<T1>* theIter;
#if defined(ENV_OSTORE)
  theIter = new (theLocality,
   OTC_KeyCursor< T1,OTC_PairBucket<T1,T2> >::get_os_typespec())
   OTC_KeyCursor< T1,OTC_PairBucket<T1,T2> >(listByItems_,
   theStartLink,theEndLink,theDirection,theProtection);
#else
  theIter = new OTC_KeyCursor< T1,OTC_PairBucket<T1,T2> >(
   listByItems_,theStartLink,theEndLink,theDirection,theProtection);
#endif
  OTCLIB_ASSERT_M(theIter != 0);

  if (theStartAnchor != 0)
    theStartAnchor->kill();
  if (theEndAnchor != 0)
    theEndAnchor->kill();

  return theIter;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_Cursor<T2>* OTC_TDualMap<T1,T2>::items_(
 T1 const& theKey,
 int (*theRankFn)(T1 const&, T1 const&),
 OTC_Direction theDirection,
 OTC_Protection theProtection
) const
{
  OTC_AVLTreeNode* theNode;
  OTC_AVLTreeNode* theStartNode;
  OTC_AVLTreeNode* theEndNode;

  theNode = (OTC_AVLTreeNode*)keyIndex_->root();

  theNode = rangeIndexKey_(theNode,theKey,theRankFn,theStartNode,theEndNode);

  if (theNode == 0)
    return 0;

  OTC_Linkable* theStartLink;
  OTC_Linkable* theEndLink;
  OTC_AVLLinkNode* theStartLinkNode;
  OTC_AVLLinkNode* theEndLinkNode;

  theStartLinkNode = (OTC_AVLLinkNode*)theStartNode->index()->first();
  theEndLinkNode = (OTC_AVLLinkNode*)theEndNode->index()->last();

  theStartLink = theStartLinkNode->link()->prev();
  theEndLink = theEndLinkNode->link()->next();

#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
  if (!theLocality.isTransientSegment())
  {
    os_transaction* theTXN = os_transaction::get_current();
    if (theTXN != 0 && theTXN->get_type() == os_transaction::read_only)
      theProtection = OTCLIB_UNSAFE;

    if (theProtection == OTCLIB_UNSAFE)
      theLocality = os_segment::get_transient_segment();
  }
#endif

  OTC_Anchor* theStartAnchor = 0;
  OTC_Anchor* theEndAnchor = 0;

  if (theProtection == OTCLIB_SAFE)
  {
#if defined(ENV_OSTORE)
    theStartAnchor = new (theLocality,OTC_Anchor::get_os_typespec())
OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new (theLocality,OTC_Anchor::get_os_typespec()) OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#else
    theStartAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#endif
    theStartLink->addAfter(theStartAnchor);
    theStartLink = theStartAnchor;
    theEndLink->addBefore(theEndAnchor);
    theEndLink = theEndAnchor;
  }

  OTC_Cursor<T2>* theIter;
#if defined(ENV_OSTORE)
  theIter = new (theLocality,
   OTC_ItemCursor< T2,OTC_PairBucket<T1,T2> >::get_os_typespec())
   OTC_ItemCursor< T2,OTC_PairBucket<T1,T2> >(listByKeys_,
   theStartLink,theEndLink,theDirection,theProtection);
#else
  theIter = new OTC_ItemCursor< T2,OTC_PairBucket<T1,T2> >(
   listByKeys_,theStartLink,theEndLink,theDirection,theProtection);
#endif
  OTCLIB_ASSERT_M(theIter != 0);

  if (theStartAnchor != 0)
    theStartAnchor->kill();
  if (theEndAnchor != 0)
    theEndAnchor->kill();

  return theIter;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_PairCursor<T1,T2>* OTC_TDualMap<T1,T2>::pairsKey_(
 T1 const& theKey,
 int (*theRankFn)(T1 const&, T1 const&),
 OTC_Direction theDirection,
 OTC_Protection theProtection
) const
{
  OTC_AVLTreeNode* theNode;
  OTC_AVLTreeNode* theStartNode;
  OTC_AVLTreeNode* theEndNode;

  theNode = (OTC_AVLTreeNode*)keyIndex_->root();

  theNode = rangeIndexKey_(theNode,theKey,theRankFn,theStartNode,theEndNode);

  if (theNode == 0)
    return 0;

  OTC_Linkable* theStartLink;
  OTC_Linkable* theEndLink;
  OTC_AVLLinkNode* theStartLinkNode;
  OTC_AVLLinkNode* theEndLinkNode;

  theStartLinkNode = (OTC_AVLLinkNode*)theStartNode->index()->first();
  theEndLinkNode = (OTC_AVLLinkNode*)theEndNode->index()->last();

  theStartLink = theStartLinkNode->link()->prev();
  theEndLink = theEndLinkNode->link()->next();

#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
  if (!theLocality.isTransientSegment())
  {
    os_transaction* theTXN = os_transaction::get_current();
    if (theTXN != 0 && theTXN->get_type() == os_transaction::read_only)
      theProtection = OTCLIB_UNSAFE;

    if (theProtection == OTCLIB_UNSAFE)
      theLocality = os_segment::get_transient_segment();
  }
#endif

  OTC_Anchor* theStartAnchor = 0;
  OTC_Anchor* theEndAnchor = 0;

  if (theProtection == OTCLIB_SAFE)
  {
#if defined(ENV_OSTORE)
    theStartAnchor = new (theLocality,OTC_Anchor::get_os_typespec())
OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new (theLocality,OTC_Anchor::get_os_typespec()) OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#else
    theStartAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#endif
    theStartLink->addAfter(theStartAnchor);
    theStartLink = theStartAnchor;
    theEndLink->addBefore(theEndAnchor);
    theEndLink = theEndAnchor;
  }

  OTC_PairCursor<T1,T2>* theIter;
#if defined(ENV_OSTORE)
  theIter = new (theLocality,
   OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >::get_os_typespec())
   OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >(listByKeys_,
   theStartLink,theEndLink,theDirection,theProtection);
#else
  theIter = new OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >(
   listByKeys_,theStartLink,theEndLink,theDirection,theProtection);
#endif
  OTCLIB_ASSERT_M(theIter != 0);

  if (theStartAnchor != 0)
    theStartAnchor->kill();
  if (theEndAnchor != 0)
    theEndAnchor->kill();

  return theIter;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_PairCursor<T1,T2>* OTC_TDualMap<T1,T2>::pairsItem_(
 T2 const& theItem,
 int (*theRankFn)(T2 const&, T2 const&),
 OTC_Direction theDirection,
 OTC_Protection theProtection
) const
{
  OTC_AVLTreeNode* theNode;
  OTC_AVLTreeNode* theStartNode;
  OTC_AVLTreeNode* theEndNode;

  theNode = (OTC_AVLTreeNode*)itemIndex_->root();

  theNode = rangeIndexItem_(theNode,theItem,theRankFn,theStartNode,theEndNode);

  if (theNode == 0)
    return 0;

  OTC_Linkable* theStartLink;
  OTC_Linkable* theEndLink;
  OTC_AVLLinkNode* theStartLinkNode;
  OTC_AVLLinkNode* theEndLinkNode;

  theStartLinkNode = (OTC_AVLLinkNode*)theStartNode->index()->first();
  theEndLinkNode = (OTC_AVLLinkNode*)theEndNode->index()->last();

  theStartLink = theStartLinkNode->link()->prev();
  theEndLink = theEndLinkNode->link()->next();

#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
  if (!theLocality.isTransientSegment())
  {
    os_transaction* theTXN = os_transaction::get_current();
    if (theTXN != 0 && theTXN->get_type() == os_transaction::read_only)
      theProtection = OTCLIB_UNSAFE;

    if (theProtection == OTCLIB_UNSAFE)
      theLocality = os_segment::get_transient_segment();
  }
#endif

  OTC_Anchor* theStartAnchor = 0;
  OTC_Anchor* theEndAnchor = 0;

  if (theProtection == OTCLIB_SAFE)
  {
#if defined(ENV_OSTORE)
    theStartAnchor = new (theLocality,OTC_Anchor::get_os_typespec())
OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new (theLocality,OTC_Anchor::get_os_typespec()) OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#else
    theStartAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theStartAnchor != 0);
    theEndAnchor = new OTC_Anchor;
    OTCLIB_ASSERT_M(theEndAnchor != 0);
#endif
    theStartLink->addAfter(theStartAnchor);
    theStartLink = theStartAnchor;
    theEndLink->addBefore(theEndAnchor);
    theEndLink = theEndAnchor;
  }

  OTC_PairCursor<T1,T2>* theIter;
#if defined(ENV_OSTORE)
  theIter = new (theLocality,
   OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >::get_os_typespec())
   OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >(listByItems_,
   theStartLink,theEndLink,theDirection,theProtection);
#else
  theIter = new OTC_KeyItemCursor< T1,T2,OTC_PairBucket<T1,T2> >(
   listByItems_,theStartLink,theEndLink,theDirection,theProtection);
#endif
  OTCLIB_ASSERT_M(theIter != 0);

  if (theStartAnchor != 0)
    theStartAnchor->kill();
  if (theEndAnchor != 0)
    theEndAnchor->kill();

  return theIter;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
T1 const& OTC_TDualMap<T1,T2>::pickKey() const
{
  OTCLIB_ENSURE_FN((population() != 0),
   "OTC_TDualMap<T1,T2>::pickKey()",
   "collection must not be empty");

  return ((OTC_PairBucket<T1,T2>*)listByKeys_->first())->key();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
T2 const& OTC_TDualMap<T1,T2>::pickItem() const
{
  OTCLIB_ENSURE_FN((population() != 0),
   "OTC_TDualMap<T1,T2>::pickItem()",
   "collection must not be empty");

  return ((OTC_PairBucket<T1,T2>*)listByItems_->first())->item();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
void OTC_TDualMap<T1,T2>::removeKey(
 T1 const& theKey,
 OTC_Protection theProtection
)
{
  // Trying to do this ourselves is a bit
  // complicated, so use an iterator to get
  // pairs and do one at a time.

  OTC_PairIterator<T1,T2> thePairs;
  thePairs = pairsKey(theKey);

  OTCLIB_ENSURE_FN((theProtection == OTCLIB_UNSAFE || thePairs.isValid()),
   "OTC_TDualMap<T1,T2>::removeKey(T1 const&, OTC_Protection)",
   "key doesn't exist");

  while (thePairs.isValid())
  {
    remove(thePairs.key(),thePairs.item());
    thePairs.next();
  }
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
void OTC_TDualMap<T1,T2>::removeItem(
 T2 const& theItem,
 OTC_Protection theProtection
)
{
  // Trying to do this ourselves is a bit
  // complicated, so use an iterator to get
  // pairs and do one at a time.

  OTC_PairIterator<T1,T2> thePairs;
  thePairs = pairsItem(theItem);

  OTCLIB_ENSURE_FN((theProtection == OTCLIB_UNSAFE || thePairs.isValid()),
   "OTC_TDualMap<T1,T2>::removeItem(T2 const&, OTC_Protection)",
   "item doesn't exist");

  while (thePairs.isValid())
  {
    remove(thePairs.key(),thePairs.item());
    thePairs.next();
  }
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
void OTC_TDualMap<T1,T2>::remove(
 T1 const& theKey,
 T2 const& theItem,
 OTC_Protection theProtection
)
{
  int theRank;
  OTC_Link* theLink;
  OTC_AVLLinkNode* theLinkNode;
  OTC_AVLTreeNode* theIndexNode;

  theIndexNode = (OTC_AVLTreeNode*)keyIndex_->root();

  theRank = searchIndexKey_(theIndexNode,theKey,theIndexNode);

  OTCLIB_ENSURE_FN((theProtection == OTCLIB_UNSAFE || theRank == 0),
   "OTC_TDualMap<T1,T2>::remove(T1 const&, T2 const&, OTC_Protection)",
   "key doesn't exist");

  if (theRank != 0)
    return;

  theLinkNode = (OTC_AVLLinkNode*)theIndexNode->index()->root();

  theRank = searchGroupItem_(theLinkNode,theItem,theLinkNode);

  OTCLIB_ENSURE_FN((theProtection == OTCLIB_UNSAFE || theRank == 0),
   "OTC_TDualMap<T1,T2>::remove(T1 const&, T2 const&, OTC_Protection)",
   "key/item pair doesn't exist");

  if (theRank != 0)
    return;

  theLink = theLinkNode->link();
  theLink->kill();

  theLinkNode->unlink();
  delete theLinkNode;

  if (theIndexNode->index()->isEmpty())
  {
    theIndexNode->unlink();
    delete theIndexNode;
  }

  // Now do it all again for item map.

  theIndexNode = (OTC_AVLTreeNode*)itemIndex_->root();

  theRank = searchIndexItem_(theIndexNode,theItem,theIndexNode);

  OTCLIB_ENSURE_FN((theRank == 0),
   "OTC_TDualMap<T1,T2>::remove(T1 const&, T2 const&, OTC_Protection)",
   "internal error");

  theLinkNode = (OTC_AVLLinkNode*)theIndexNode->index()->root();

  theRank = searchGroupKey_(theLinkNode,theKey,theLinkNode);

  OTCLIB_ENSURE_FN((theRank == 0),
   "OTC_TDualMap<T1,T2>::remove(T1 const&, T2 const&, OTC_Protection)",
   "internal error");

  theLink = theLinkNode->link();
  theLink->kill();

  theLinkNode->unlink();
  delete theLinkNode;

  if (theIndexNode->index()->isEmpty())
  {
    theIndexNode->unlink();
    delete theIndexNode;
  }

  count_->dec();
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
int OTC_TDualMap<T1,T2>::rankKey(T1 const& key1, T1 const& key2) const
{
  return (keyRankFn_ != 0) ? (*keyRankFn_)(key1,key2) :
   OTC_RankActions<T1>::rank(key1,key2);
}
 
/* ------------------------------------------------------------------------- */
template<class T1, class T2>
int OTC_TDualMap<T1,T2>::rankItem(T2 const& item1, T2 const& item2) const
{
  return (itemRankFn_ != 0) ? (*itemRankFn_)(item1,item2) :
   OTC_RankActions<T2>::rank(item1,item2);
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_AVLTreeNode* OTC_TDualMap<T1,T2>::rangeIndexKey_(
 OTC_AVLTreeNode* theRoot,
 T1 const& theKey,
 int (*theRankFn)(T1 const&, T1 const&),
 OTC_AVLTreeNode*& theStartNode,
 OTC_AVLTreeNode*& theEndNode
) const
{
  int theRank;
  OTC_AVLTreeNode* tmpIndexNode;
  OTC_AVLTreeNode* theIndexNode;

  theStartNode = 0;
  theEndNode = 0;

  tmpIndexNode = theRoot;

  while (tmpIndexNode != 0)
  {
    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)
     ((OTC_AVLLinkNode*)tmpIndexNode->index()->root())->link();

    if (theRankFn != 0)
      theRank = (*theRankFn)(theKey,tmpBucket->key());
    else
      theRank = rankKey(theKey,tmpBucket->key());

    if (theRank == 0)
      break;
    else if (theRank < 0)
      tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->left();
    else
      tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->right();
  }

  if (tmpIndexNode == 0)
    return 0;

  theIndexNode = tmpIndexNode;

  OTC_AVLTreeNode* tmpStartNode;
  OTC_AVLTreeNode* tmpEndNode;

  tmpStartNode = theIndexNode;
  tmpIndexNode = (OTC_AVLTreeNode*)theIndexNode->prev();

  while (tmpIndexNode != 0)
  {
    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)
     ((OTC_AVLLinkNode*)tmpIndexNode->index()->root())->link();

    if (theRankFn != 0)
      theRank = (*theRankFn)(theKey,tmpBucket->key());
    else
      theRank = rankKey(theKey,tmpBucket->key());

    if (theRank != 0)
      break;

    tmpStartNode = tmpIndexNode;
    tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->prev();
  }

  tmpEndNode = theIndexNode;
  tmpIndexNode = (OTC_AVLTreeNode*)theIndexNode->next();

  while (tmpIndexNode != 0)
  {
    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)
     ((OTC_AVLLinkNode*)tmpIndexNode->index()->root())->link();

    if (theRankFn != 0)
      theRank = (*theRankFn)(theKey,tmpBucket->key());
    else
      theRank = rankKey(theKey,tmpBucket->key());

    if (theRank != 0)
      break;

    tmpEndNode = tmpIndexNode;
    tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->next();
  }

  theStartNode = tmpStartNode;
  theEndNode = tmpEndNode;

  return theIndexNode;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_AVLTreeNode* OTC_TDualMap<T1,T2>::rangeIndexItem_(
 OTC_AVLTreeNode* theRoot,
 T2 const& theItem,
 int (*theRankFn)(T2 const&, T2 const&),
 OTC_AVLTreeNode*& theStartNode,
 OTC_AVLTreeNode*& theEndNode
) const
{
  int theRank;
  OTC_AVLTreeNode* tmpIndexNode;
  OTC_AVLTreeNode* theIndexNode;

  theStartNode = 0;
  theEndNode = 0;

  tmpIndexNode = theRoot;

  while (tmpIndexNode != 0)
  {
    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)
     ((OTC_AVLLinkNode*)tmpIndexNode->index()->root())->link();

    if (theRankFn != 0)
      theRank = (*theRankFn)(theItem,tmpBucket->item());
    else
      theRank = rankItem(theItem,tmpBucket->item());

    if (theRank == 0)
      break;
    else if (theRank < 0)
      tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->left();
    else
      tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->right();
  }

  if (tmpIndexNode == 0)
    return 0;

  theIndexNode = tmpIndexNode;

  OTC_AVLTreeNode* tmpStartNode;
  OTC_AVLTreeNode* tmpEndNode;

  tmpStartNode = theIndexNode;
  tmpIndexNode = (OTC_AVLTreeNode*)theIndexNode->prev();

  while (tmpIndexNode != 0)
  {
    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)
     ((OTC_AVLLinkNode*)tmpIndexNode->index()->root())->link();

    if (theRankFn != 0)
      theRank = (*theRankFn)(theItem,tmpBucket->item());
    else
      theRank = rankItem(theItem,tmpBucket->item());

    if (theRank != 0)
      break;

    tmpStartNode = tmpIndexNode;
    tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->prev();
  }

  tmpEndNode = theIndexNode;
  tmpIndexNode = (OTC_AVLTreeNode*)theIndexNode->next();

  while (tmpIndexNode != 0)
  {
    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)
     ((OTC_AVLLinkNode*)tmpIndexNode->index()->root())->link();

    if (theRankFn != 0)
      theRank = (*theRankFn)(theItem,tmpBucket->item());
    else
      theRank = rankItem(theItem,tmpBucket->item());

    if (theRank != 0)
      break;

    tmpEndNode = tmpIndexNode;
    tmpIndexNode = (OTC_AVLTreeNode*)tmpIndexNode->next();
  }

  theStartNode = tmpStartNode;
  theEndNode = tmpEndNode;

  return theIndexNode;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
int OTC_TDualMap<T1,T2>::searchIndexKey_(
 OTC_AVLTreeNode* theRoot,
 T1 const& theKey,
 OTC_AVLTreeNode*& theNode
) const
{
  theNode = 0;

  OTC_AVLNode* tmpNode = 0;
  OTC_AVLNode* tmpParent = 0;
  int theRank = 0;

  if (theRoot == 0)
    return 1;

  tmpNode = theRoot;

  while (tmpNode != 0)
  {
    tmpParent = tmpNode;

    OTC_AVLNode* tmpRoot;
    tmpRoot = ((OTC_AVLTreeNode*)tmpNode)->index()->root();
    OTCLIB_ASSERT(tmpRoot != 0);

    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)((OTC_AVLLinkNode*)tmpRoot)->link();

    theRank = rankKey(theKey,tmpBucket->key());
    if (theRank == 0)
      break;
    else if (theRank < 0)
      tmpNode = tmpNode->left();
    else
      tmpNode = tmpNode->right();
  }

  if (tmpNode != 0)
  {
    theNode = (OTC_AVLTreeNode*)tmpNode;
    return 0;
  }

  theNode = (OTC_AVLTreeNode*)tmpParent;
  return theRank;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
int OTC_TDualMap<T1,T2>::searchIndexItem_(
 OTC_AVLTreeNode* theRoot,
 T2 const& theItem,
 OTC_AVLTreeNode*& theNode
) const
{
  theNode = 0;

  OTC_AVLNode* tmpNode = 0;
  OTC_AVLNode* tmpParent = 0;
  int theRank = 0;

  if (theRoot == 0)
    return 1;

  tmpNode = theRoot;

  while (tmpNode != 0)
  {
    tmpParent = tmpNode;

    OTC_AVLNode* tmpRoot;
    tmpRoot = ((OTC_AVLTreeNode*)tmpNode)->index()->root();
    OTCLIB_ASSERT(tmpRoot != 0);

    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)((OTC_AVLLinkNode*)tmpRoot)->link();

    theRank = rankItem(theItem,tmpBucket->item());
    if (theRank == 0)
      break;
    else if (theRank < 0)
      tmpNode = tmpNode->left();
    else
      tmpNode = tmpNode->right();
  }

  if (tmpNode != 0)
  {
    theNode = (OTC_AVLTreeNode*)tmpNode;
    return 0;
  }

  theNode = (OTC_AVLTreeNode*)tmpParent;
  return theRank;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
int OTC_TDualMap<T1,T2>::searchGroupItem_(
 OTC_AVLLinkNode* theRoot,
 T2 const& theItem,
 OTC_AVLLinkNode*& theNode
) const
{
  theNode = 0;

  OTC_AVLNode* tmpNode = 0;
  OTC_AVLNode* tmpParent = 0;
  int theRank = 0;

  if (theRoot == 0)
    return 1;

  tmpNode = theRoot;

  while (tmpNode != 0)
  {
    tmpParent = tmpNode;

    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)((OTC_AVLLinkNode*)tmpNode)->link();

    theRank = rankItem(theItem,tmpBucket->item());
    if (theRank == 0)
      break;
    else if (theRank < 0)
      tmpNode = tmpNode->left();
    else
      tmpNode = tmpNode->right();
  }

  if (tmpNode != 0)
  {
    theNode = (OTC_AVLLinkNode*)tmpNode;
    return 0;
  }

  theNode = (OTC_AVLLinkNode*)tmpParent;
  return theRank;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
int OTC_TDualMap<T1,T2>::searchGroupKey_(
 OTC_AVLLinkNode* theRoot,
 T1 const& theKey,
 OTC_AVLLinkNode*& theNode
) const
{
  theNode = 0;

  OTC_AVLNode* tmpNode = 0;
  OTC_AVLNode* tmpParent = 0;
  int theRank = 0;

  if (theRoot == 0)
    return 1;

  tmpNode = theRoot;

  while (tmpNode != 0)
  {
    tmpParent = tmpNode;

    OTC_PairBucket<T1,T2>* tmpBucket;
    tmpBucket = (OTC_PairBucket<T1,T2>*)((OTC_AVLLinkNode*)tmpNode)->link();

    theRank = rankKey(theKey,tmpBucket->key());
    if (theRank == 0)
      break;
    else if (theRank < 0)
      tmpNode = tmpNode->left();
    else
      tmpNode = tmpNode->right();
  }

  if (tmpNode != 0)
  {
    theNode = (OTC_AVLLinkNode*)tmpNode;
    return 0;
  }

  theNode = (OTC_AVLLinkNode*)tmpParent;
  return theRank;
}

/* ------------------------------------------------------------------------- */
template<class T1, class T2>
OTC_Link* OTC_TDualMap<T1,T2>::link_(T1 const& theKey, T2 const& theItem)
{
#if defined(ENV_OSTORE)
  OTC_Locality theLocality = OTC_Locality::of(this);
#endif

  OTC_PairBucket<T1,T2>* theBucket;
#if defined(ENV_OSTORE)
  theBucket = new (theLocality,OTC_PairBucket<T1,T2>::get_os_typespec())
   OTC_PairBucket<T1,T2>(theKey,theItem);
#else
  theBucket = new OTC_PairBucket<T1,T2>(theKey,theItem);
#endif
  OTCLIB_ASSERT_M(theBucket != 0);

  return theBucket;
}

/* ------------------------------------------------------------------------- */

#endif /* OTC_COLLCTN_TDUALMAP_C */
