#ifndef OTC_PYTHON_PYQUEUE_HH
#define OTC_PYTHON_PYQUEUE_HH
/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     python/pyqueue.hh
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1998-1999 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#include <OTC/tcl/tclqueue.hh>

#ifdef __GNUG__
#pragma interface "OTC/python/pyqueue.hh"
#endif

/* ------------------------------------------------------------------------- */

class OSE_EXPORT OTC_PyJobQueue : public OTC_TclJobQueue
    // = TITLE
    //     Dispatcher job queue suitable for use with Python.
    //
    // = DESCRIPTION
    //     The <OTC_PyJobQueue> class derives from <OTC_TclJobQueue>. It does
    //     this so it can override the <shutdown()> method to more gracefully
    //     shutdown the program main loop. In the base class when the
    //     dispatcher is stopped <Tcl_Exit()> is called. This has the effect
    //     of exiting the program immediately. Instead, this derived class
    //     will call "_tkinter.quit()". Doing this will cause the main
    //     program loop to stop if it was the case that "_tkinter.mainloop()"
    //     was invoked in the first place. If the main program loop was
    //     started using the <run()> method of the <OTC_Dispatcher> class,
    //     then it will stop anyway.
{
  public:

			~OTC_PyJobQueue();

    // = INITIALISATION

			OTC_PyJobQueue();
				// Does nothing. Everything is done by
				// the base classes.

    // = SHUTDOWN

    void		shutdown();
				// This is called when it is detected that
				// the dispatcher has been stopped due to
				// <OTC_Dispatcher::stop()> being called.
				// This function will execute Python code
				// to call "_tkinter.quit()". The call is
				// done in a "try/except" clause so that an
				// error message is not output if neither
				// the "_tkinter" or "Tkinter" modules are
				// loaded.

  private:

			OTC_PyJobQueue(OTC_PyJobQueue const&);
				// Do not define an implementation for this.

    OTC_PyJobQueue&	operator=(OTC_PyJobQueue const&);
				// Do not define an implementation for this.

};

/* ------------------------------------------------------------------------- */

#endif /* OTC_PYTHON_PYQUEUE_HH */
