#ifndef OTC_TEXT_STRNGBUF_HH
#define OTC_TEXT_STRNGBUF_HH
/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     text/strngbuf.hh
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 2000-2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#include <OTC/text/string.hh>

#ifdef __GNUG__
#pragma interface "OTC/text/strngbuf.hh"
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4275)
#endif

/* ------------------------------------------------------------------------- */

#if defined(ENV_OSTORE) && defined(CXX_DEC) && defined(__alpha)
#pragma __environment save
#pragma __environment header_defaults
#endif

class OSE_EXPORT OTC_StringBuf : public streambuf
    // = TITLE
    //     Streambuf class for use on an instance of a string class.
    //
    // = CLASS TYPE
    //     Concrete
    //
    // = DESCRIPTION
    //     The <OTC_StringBuf> class is a derived version of <streambuf>
    //     specifically for formatting output directly into an instance of
    //     the <OTC_String> class. At the same time as the stream is
    //     being used on the string, you can use all the member functions
    //     of the <OTC_String> class.
    //
    // = NOTES
    //     The referenced string must exist for the lifetime of the streambuf
    //     which refers to it.
    //     
    // = SEE ALSO
    //     <OTC_OSStream>, <OTC_String>
{
  public:

			~OTC_StringBuf();

    // = INITIALISATION

			OTC_StringBuf(OTC_String& theString);
				// <theString> should be a reference to
				// an instance of <OTC_String> to which
				// the buffer contents should be appended.

  protected:

    // These are overrides of virtual functions in the <streambuf> base
    // class.

#if defined(ENV_CXXSTDLIB)

    int_type		overflow(int_type c);

    int_type		underflow();

#else

    int			overflow(int c);

    int			underflow();

#endif

  private:

			OTC_StringBuf(OTC_StringBuf const&);
				// Do not provide an implementation for this.

    OTC_StringBuf&	operator=(OTC_StringBuf const&);
				// Do not provide an implementation for this.

    OTC_String&		string_;
    				// The string to which data is being appended.
};

#if defined(ENV_OSTORE) && defined(CXX_DEC) && defined(__alpha)
#pragma __environment restore
#endif

/* ------------------------------------------------------------------------- */

#endif /* OTC_TEXT_STRNGBUF_HH */
