/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     python/otcpydsptch.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1999-2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#include <OTC/python/pydsptch.hh>
#include <OTC/python/pyqueue.hh>
#include <OTC/python/pyjob.hh>
#include <OTC/python/pymodule.hh>

#include <OTC/dispatch/dispatch.hh>
#include <OTC/dispatch/shutdown.hh>

/* ------------------------------------------------------------------------- */
OTC_PyDispatcher* OTC_PyDispatcher::gInstance_ = 0;
OTC_PyJobQueue* OTC_PyDispatcher::gJobQueue_ = 0;
PyThreadState* OTC_PyDispatcher::gThreadState_ = 0;

/* ------------------------------------------------------------------------- */
PyTypeObject OTC_PyDispatcher::gType_ = {
  PyObject_HEAD_INIT(&PyType_Type)
  0,
  "Dispatcher",
  sizeof(OTC_PyDispatcher),
  0,
  OTC_PyDispatcher::sys_dealloc,
  0,
  OTC_PyDispatcher::sys_getattr,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0
};

/* ------------------------------------------------------------------------- */
PyMethodDef OTC_PyDispatcher::gMethods_[] = {
  { "run", OTC_PyDispatcher::mfn_run, 1 },
  { "stop", OTC_PyDispatcher::mfn_stop, 1 },
  { "schedule", OTC_PyDispatcher::mfn_schedule, 1 },
  { "monitor", OTC_PyDispatcher::mfn_monitor, 1 },
  { "shutdown", OTC_PyDispatcher::mfn_shutdown, 1 },
  { 0, 0 }
};

/* ------------------------------------------------------------------------- */
OTC_PyDispatcher::~OTC_PyDispatcher()
{
  // Nothing to do.
}

/* ------------------------------------------------------------------------- */
OTC_PyDispatcher::OTC_PyDispatcher()
{
  ob_type = &gType_;

  _Py_NewReference(this);
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::instance(PyObject* theModule, PyObject* theArgs)
{
  if (!PyArg_ParseTuple(theArgs,""))
    return 0;

  if (gJobQueue_ == 0)
  {
    gJobQueue_ = new OTC_PyJobQueue;
    OTCLIB_ASSERT_M(gJobQueue_ != 0);

    OTC_Dispatcher::initialise(gJobQueue_);
  }

  if (gInstance_ == 0)
  {
    gInstance_ = new OTC_PyDispatcher;
    OTCLIB_ASSERT_M(gInstance_ != 0);

#ifdef WITH_THREAD
    gThreadState_ = PyThreadState_Get();
#endif
  }

  Py_XINCREF(gInstance_);

  return (PyObject*)gInstance_;
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::obj_getattr(char const* theName)
{
  return Py_FindMethod(gMethods_,this,(char*)theName);
}

/* ------------------------------------------------------------------------- */
void OTC_PyDispatcher::sys_dealloc(PyObject* theInstance)
{
  delete (OTC_PyDispatcher*)theInstance;
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::sys_getattr(
 PyObject* theInstance,
 char* theName
)
{
  return ((OTC_PyDispatcher*)theInstance)->obj_getattr(theName);
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::mfn_run(PyObject*, PyObject*)
{
  // Py_BEGIN_ALLOW_THREADS

  OTC_Dispatcher::run();

  // Py_END_ALLOW_THREADS

  Py_XINCREF(Py_None);

  return Py_None;
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::mfn_stop(PyObject*, PyObject*)
{
  OTC_Dispatcher::stop();

  Py_XINCREF(Py_None);

  return Py_None;
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::mfn_schedule(PyObject*, PyObject* theArgs)
{
  PyObject* theCallback;
  int theOptions = OTCLIB_STANDARD_JOB;

  if (!PyArg_ParseTuple(theArgs,"O|i",&theCallback,&theOptions))
    return 0;

  OTC_Job* theJob;
  theJob = new OTC_PyJob(theCallback);
  OTCLIB_ASSERT(theJob != 0);

  OTC_Dispatcher::schedule(theJob,theOptions);

  Py_XINCREF(Py_None);

  return Py_None;
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::mfn_monitor(PyObject*, PyObject* theArgs)
{
  int theSignal;

  if (!PyArg_ParseTuple(theArgs,"i",&theSignal))
    return 0;

  OTCEV_Shutdown::monitor(theSignal);

  Py_XINCREF(Py_None);

  return Py_None;
}

/* ------------------------------------------------------------------------- */
PyObject* OTC_PyDispatcher::mfn_shutdown(PyObject*, PyObject* theArgs)
{
  double theDelay = 0;

  if (!PyArg_ParseTuple(theArgs,"|d",&theDelay))
    return 0;

  OTCEV_Shutdown::schedule(int(theDelay*1000));

  Py_XINCREF(Py_None);

  return Py_None;
}

/* ------------------------------------------------------------------------- */
