/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     dispatch/otcresponse.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 2000-2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/dispatch/failure.hh"
#endif

#include <OTC/dispatch/failure.hh>

#include <OTC/dispatch/message.hh>
#include <OTC/dispatch/svpyload.hh>

/* ------------------------------------------------------------------------- */
int OTCEV_Failure::gTypeId_ = 0;

/* ------------------------------------------------------------------------- */
OTCEV_Failure::~OTCEV_Failure()
{
  envelope_->destroy();
}

/* ------------------------------------------------------------------------- */
OTCEV_Failure::OTCEV_Failure(OTCEV_Envelope* theEnvelope)
  : envelope_(0), error_(-1)
{
  OTCLIB_ENSURE_FN((theEnvelope != 0),
   "OTCEV_Failure::OTCEV_Failure(OTCEV_Envelope*)",
   "invalid envelope");

  envelope_ = (OTCEV_Envelope*)theEnvelope->clone();

  OTC_SVPayload theContent;
  theContent.decode(envelope_->message()->content());

  OTC_Boolean theStatus;
  theStatus = OTCLIB_TRUE;

  (theContent["error"],theStatus) >>= error_;
  (theContent["description"],theStatus) >>= description_;
  (theContent["origin"],theStatus) >>= origin_;
  (theContent["details"],theStatus) >>= details_;

  OTCLIB_ENSURE_FN_W((theStatus != OTCLIB_FALSE),
   "OTCEV_Failure::OTCEV_Failure(...)",
   "Malformed failure notification.");
}

/* ------------------------------------------------------------------------- */
void* OTCEV_Failure::type() const
{
  return typeId();
}

/* ------------------------------------------------------------------------- */
void OTCEV_Failure::dump(ostream& outs) const
{
  outs << "<OTC> FAILURE - recipient = " << envelope_->to();
  outs << ", sender = " << envelope_->from();
  outs << ", id = " << envelope_->conversationId();
  outs << ", seq = " << envelope_->sequenceNumber();
  outs << ", error = " << error_;
  outs << ", description = " << description_;
  outs << ", origin = " << origin_;
  outs << ", details = " << details_;
}

/* ------------------------------------------------------------------------- */
