/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     memory/otccmmnpool.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1993 OTC LIMITED
//     Copyright 1994-1997 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/memory/cmmnpool.hh"
#endif

#include <OTC/memory/cmmnpool.hh>
#include <OTC/memory/pool.hh>
#include <OTC/thread/mxreaper.hh>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* ------------------------------------------------------------------------- */
OTC_NRMutex OTC_CommonPool::mutex_;
OTC_Boolean OTC_CommonPool::init_ = OTCLIB_FALSE;
OTC_Boolean OTC_CommonPool::disable_ = OTCLIB_FALSE;
OTC_Pool** OTC_CommonPool::pools_ = 0;
u_int OTC_CommonPool::numPools_ = 0;
size_t OTC_CommonPool::increment_ = 0;
size_t OTC_CommonPool::largest_ = 0;
u_int* OTC_CommonPool::index_ = 0;

/* ------------------------------------------------------------------------- */
void OTC_CommonPool::initialise()
{
  // This function does not lock the thread mutex as it is expected that it
  // is locked by the allocate() function. Avoids having a recursive mutex.

  init_ = OTCLIB_TRUE;

  if (getenv("OTCLIB_NOCOMMONPOOL") != 0)
    disable_ = OTCLIB_TRUE;

  if (disable_ == OTCLIB_FALSE && pools_ == 0)
  {
    // Initialise statics here. Do this in order to avoid problems with
    // initialised static data in SunOS 4 Shared libraries.

    char* theVariable = 0;

    theVariable = getenv("OTCLIB_COMMONPOOLS");
    if (theVariable != 0)
    {
      int theNumber;
      theNumber = atoi(theVariable);
      if (theNumber <= 0)
	disable_ = OTCLIB_TRUE;
      else
	numPools_ = theNumber;
    }
    else
    {
      numPools_ = 8;
    }

    increment_ = sizeof(double);

    largest_ = numPools_ * increment_;

    pools_ = new OTC_Pool*[numPools_];
    OTCLIB_ASSERT_M(pools_ != 0);

    char tmpBuffer1[64];
    char tmpBuffer2[64];

    strcpy(tmpBuffer1,"OTCLIB_POOLBLOCKSIZE_");
    strcpy(tmpBuffer2,"OTCLIB_POOLPREALLOCATE_");

    char* tmpPointer1 = tmpBuffer1 + strlen(tmpBuffer1);
    char* tmpPointer2 = tmpBuffer2 + strlen(tmpBuffer2);

    size_t theSize = increment_;
    for (u_int i=0; i<numPools_; i++)
    {
      int theBlockSize = -1;
      int thePreAllocate = -1;

      sprintf(tmpPointer1,"%d",i+1);
      theVariable = getenv(tmpBuffer1);
      if (theVariable != 0)
	theBlockSize = atoi(theVariable);

      sprintf(tmpPointer2,"%d",i+1);
      theVariable = getenv(tmpBuffer2);
      if (theVariable != 0)
	thePreAllocate = atoi(theVariable);

      if (theBlockSize > 0)
	pools_[i] = new OTC_Pool(theSize,theBlockSize);
      else
	pools_[i] = new OTC_Pool(theSize);

      OTCLIB_ASSERT_M(pools_[i] != 0);

      if (thePreAllocate != -1)
	pools_[i]->grabMemory(thePreAllocate);

      theSize += increment_;
    }

    index_ = new u_int[largest_+1];
    OTCLIB_ASSERT_M(index_ != 0);

    index_[0] = 0;
    for (u_int j=1; j<=largest_; j++)
      index_[j] = (j-1)/increment_;
  }
}

/* ------------------------------------------------------------------------- */
void* OTC_CommonPool::allocate(size_t theSize)
{
  void* theData;

  mutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex;
  xxxMutex.grab(mutex_);

  if (init_ == OTCLIB_FALSE)
    initialise();

  if (!disable_ && (theSize <= largest_))
    theData = pools_[index_[theSize]]->allocate();
  else
    theData = new char[theSize];

  xxxMutex.release();

  mutex_.unlock();

  return theData;
}

/* ------------------------------------------------------------------------- */
void OTC_CommonPool::release(void* theMemory, size_t theSize)
{
  mutex_.lock();

  OTC_MutexReaper<OTC_NRMutex> xxxMutex;
  xxxMutex.grab(mutex_);

  if (!disable_ && (theSize <= largest_))
    pools_[index_[theSize]]->release(theMemory);
  else
    delete [] (char*)theMemory;

  xxxMutex.release();

  mutex_.unlock();
}

/* ------------------------------------------------------------------------- */
