/*
// ============================================================================
//
// = LIBRARY
//     OTC
// 
// = FILENAME
//     message/otcinetsrvr.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
// 
// = COPYRIGHT
//     Copyright 1995-2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/message/inetsrvr.hh"
#endif

#include <OTC/message/inetsrvr.hh>
#include <OTC/message/inetlstn.hh>
#include <OTC/message/episockt.hh>
#include <OTC/debug/tracer.hh>
#include <OTC/debug/tracetag.hh>

#include <string.h>

#if !defined(SYS_UNIX)
#include <windows.h>
#include <winsock.h>
#include <io.h>
#endif

#ifndef INVALID_SOCKET
#define INVALID_SOCKET (-1)
#endif

#ifndef SOCKET_ERROR
#define SOCKET_ERROR (-1)
#endif

/* ------------------------------------------------------------------------- */
static OTC_NRMutex MODULE_TRACETAG_MUTEX;
static OTC_TraceTag& MODULE_TRACETAG()
{
  static OTC_TraceTag* TAG = 0;
  MODULE_TRACETAG_MUTEX.lock();
  if (TAG == 0)
    TAG = new OTC_TraceTag("OTC_InetServer");
  MODULE_TRACETAG_MUTEX.unlock();
  return *TAG;
}

#define MODULE MODULE_TRACETAG()

/* ------------------------------------------------------------------------- */
OTC_InetServer::OTC_InetServer(
 char const* theInterface,
 OTC_EPListener* theListener,
 int theFd
)
  : OTC_EPServer(theInterface,"INET",theListener),
    fd_(theFd)
{
  OTCLIB_MARKBLOCK(MODULE,
   "OTC_InetServer::OTC_InetServer(...)");

  OTCLIB_ASSERT(theFd > 0);

  handler_ = OTC_EPISocket::create(theInterface,this);

  attach(handler_);
}

/* ------------------------------------------------------------------------- */
OTC_String OTC_InetServer::localTransport() const
{
  return localTransport_;
}

/* ------------------------------------------------------------------------- */
OTC_String OTC_InetServer::remoteTransport() const
{
  return remoteTransport_;
}

/* ------------------------------------------------------------------------- */
void OTC_InetServer::start()
{
  if (fd_ != INVALID_SOCKET)
  {
    OTC_InetAddress theAddress;

    if (getsockname(fd_,theAddress.addr(),
     &theAddress.addrlen()) != SOCKET_ERROR)
    {
      localTransport_ = theAddress.numericAddress();
    }

    if (getpeername(fd_,theAddress.addr(),
     &theAddress.addrlen()) != SOCKET_ERROR)
    {
      remoteTransport_ = theAddress.numericAddress();
    }
  }

  handler_->attach(fd_);
}

/* ------------------------------------------------------------------------- */
void OTC_InetServer::suspendInput()
{
  OTCLIB_MARKBLOCK(MODULE,"OTC_InetServer::suspendInput()");

  handler_->suspendInput();
}

/* ------------------------------------------------------------------------- */
void OTC_InetServer::resumeInput()
{
  OTCLIB_MARKBLOCK(MODULE,"OTC_InetServer::resumeInput()");

  handler_->resumeInput();
}

/* ------------------------------------------------------------------------- */
void OTC_InetServer::close()
{
  OTCLIB_MARKBLOCK(MODULE,"OTC_InetServer::close()");

  if (handler_->fd() != INVALID_SOCKET)
  {
    int theFd;
    theFd = handler_->detach();
    shutdown((void*)&theFd,0);
  }
}

/* ------------------------------------------------------------------------- */
void* OTC_InetServer::shutdown(void* theHandle, int)
{
  OTCLIB_MARKBLOCK(MODULE,"OTC_InetServer::shutdown(void*,int)");

  int theFd = INVALID_SOCKET;

  if (theHandle != 0)
    theFd = *(int*)theHandle;

  if (theFd != INVALID_SOCKET)
  {
#if defined(SYS_UNIX)
    ::close(theFd);
#else
    closesocket(theFd);
#endif
  }

  return 0;
}

/* ------------------------------------------------------------------------- */
OTC_InetServer::~OTC_InetServer()
{
  OTCLIB_MARKBLOCK(MODULE,"OTC_InetServer::~OTC_InetServer()");

  // Nothing to do.
}

/* ------------------------------------------------------------------------- */
