/*
// ============================================================================
//
// = LIBRARY
//     OTC
//
// = FILENAME
//     message/otcunixaddr.cc
//
// = AUTHOR(S)
//     Graham Dumpleton
//
// = COPYRIGHT
//     Copyright 1991 OTC LIMITED
//     Copyright 1995-1999 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
//
// ============================================================================
*/

#ifdef __GNUG__
#pragma implementation "OTC/message/unixaddr.hh"
#endif
 
#include <OTC/message/unixaddr.hh>

#if defined(SYS_UNIX) && !defined(SYS_X86_CYGWIN32)

#include <string.h>

/* ------------------------------------------------------------------------- */
char const* const OTC_UnixAddress::ANY = "";

/* ------------------------------------------------------------------------- */
OTC_UnixAddress::OTC_UnixAddress()
{
  resetAddress();
}

/* ------------------------------------------------------------------------- */
OTC_UnixAddress::OTC_UnixAddress(char const* pathname)
{
  OTCLIB_ASSERT(pathname != 0);
  initAddress(pathname);
}

/* ------------------------------------------------------------------------- */
OTC_String OTC_UnixAddress::pathname() const
{
  OTC_String aString(address_.sun_path);
  return aString;
}

/* ------------------------------------------------------------------------- */
sockaddr* OTC_UnixAddress::addr()
{
  return (sockaddr*)(void*)&address_;
}

/* ------------------------------------------------------------------------- */
sockaddr const* OTC_UnixAddress::addr() const
{
  return (sockaddr const*)(void const*)&address_;
}

/* ------------------------------------------------------------------------- */
void OTC_UnixAddress::resetAddress()
{
  length_ = sizeof(address_);
  memset(&address_,0,length_);
  address_.sun_family = AF_UNIX;
  address_.sun_path[0] = EOS;
  length_ = sizeof(address_.sun_family);
}

/* ------------------------------------------------------------------------- */
void OTC_UnixAddress::initAddress(char const* pathname)
{
  u_int pathlen = strlen(pathname);

  OTCLIB_ASSERT(pathlen<((sizeof(address_)-sizeof(address_.sun_family))));

  resetAddress();
  if (*pathname == EOS)
    strcpy(address_.sun_path,tmpnam(0));
  else
    strcpy(address_.sun_path,pathname);
  length_ = pathlen + sizeof(address_.sun_family);
}

/* ------------------------------------------------------------------------- */

#endif
