# COPYIGHT 2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED
#
# This software is made available under the terms of the Q Public License
# version 1.0. A copy of the license and further information regarding
# this package can be found at "http://ose.sourceforge.net".

OSE_RELEASE = "@OSE_RELEASE_FULL_NAME@"

import string
import types
import xmllib
import base64
import urllib
import httplib
import traceback
import operator
import socket
import time
import cgi
import sys

try:
  import sgmlop
except ImportError:
  sgmlop = None


# Function which returns a detailed description of the current Python
# exception. This function should only be called from within the context
# of a Python "except" clause.

def exceptionDescription():
  info = sys.exc_info()[:2]
  exc_type = info[0]
  exc_value = info[1]
  return str(exc_type) + ": " + str(exc_value)

def exceptionDetails(skip=0):
  info = sys.exc_info()
  exc_type = info[0]
  exc_value = info[1]
  exc_traceback = info[2]
  trace = traceback.extract_tb(exc_traceback)
  trace = trace[skip:]
  error = "Python Exception\n"
  try:
    where = "Module: " + str(callback.im_class.__module__) + "\n"
    where = where + "Method: " + str(callback.im_class.__name__)
    where = where + "." + str(callback.__name__) + "\n"
  except:
    try:
      where = "Function: " + str(callback.__name__) + "\n"
    except:
      where = ""
  error = error + where
  error = error + "Traceback (innermost last):\n"
  for filename,lineno,function,message in trace:
    error = error + "  File \"" + filename +  "\""
    error = error + ", line " + str(lineno)
    error = error + ", in " + function + "\n"
    if message:
      error = error + "    " + message + "\n"
  error = error + str(exc_type) + ": " + str(exc_value)
  return error


# Define some high level error types related to sending requests.

CLIENT_UNKNOWN_ERROR = 40
CLIENT_RESOURCE_UNAVAILABLE = 41
CLIENT_REQUEST_ENCODING_ERROR = 42
CLIENT_RESPONSE_HEADERS_INVALID = 43
CLIENT_RESPONSE_DECODING_ERROR = 44
CLIENT_SERVER_ADDRESS_INVALID = 45
CLIENT_SERVER_UNAVAILABLE = 46
CLIENT_REQUEST_TIMEOUT = 47

CLIENT_UNKNOWN_ERROR_MESSAGE = "Unknown Error"
CLIENT_RESOURCE_UNAVAILABLE_MESSAGE = "Resource Unavailable"
CLIENT_REQUEST_ENCODING_ERROR_MESSAGE = "Request Encoding Error"
CLIENT_RESPONSE_HEADERS_INVALID_MESSAGE = "Response Headers Invalid"
CLIENT_RESPONSE_DECODING_ERROR_MESSAGE = "Response Decoding Error"
CLIENT_SERVER_ADDRESS_INVALID_MESSAGE = "Server Address Invalid"
CLIENT_SERVER_UNAVAILABLE_MESSAGE = "Server Unavailable"
CLIENT_REQUEST_TIMEOUT_MESSAGE = "Request Timeout"

SERVER_UNKNOWN_ERROR = 50
SERVER_RESOURCE_UNAVAILABLE = 51
SERVER_RESPONSE_ENCODING_ERROR = 52
SERVER_REQUEST_HEADERS_INVALID = 53
SERVER_REQUEST_DECODING_ERROR = 54
SERVER_METHOD_UNAVAILABLE = 55
SERVER_PARAMETERS_INCORRECT = 56
SERVER_APPLICATION_ERROR = 57

SERVER_UNKNOWN_ERROR_MESSAGE = "Unknown Error"
SERVER_RESOURCE_UNAVAILABLE_MESSAGE = "Resource Unavailable"
SERVER_RESPONSE_ENCODING_ERROR_MESSAGE = "Response Encoding Error"
SERVER_REQUEST_HEADERS_INVALID_MESSAGE = "Request Headers Invalid"
SERVER_REQUEST_DECODING_ERROR_MESSAGE = "Request Decoding Error"
SERVER_METHOD_UNAVAILABLE_MESSAGE = "Method Unavailable"
SERVER_PARAMETERS_INCORRECT_MESSAGE = "Parameters Incorrect"
SERVER_APPLICATION_ERROR_MESSAGE = "Application Error"


class ServiceFailure(Exception):

  def __init__(self,error,description,origin="",details=""):
    self.error = error
    self.description = description
    self.origin = origin
    self.details = details

  def __repr__(self):
    return ("<ServiceFailure %s, description=%s, origin=%s, details=%s>" %
        (self.error,`self.description`,`self.origin`,`self.details`))

  __str__ = __repr__


class AddressInvalid(ServiceFailure):

  def __init__(self,error,description,origin="",details=""):
    ServiceFailure.__init__(self,error,description,origin,details)


class ServiceUnavailable(ServiceFailure):

  def __init__(self,error,description,origin="",details=""):
    ServiceFailure.__init__(self,error,description,origin,details)


class TransportFailure(ServiceFailure):

  def __init__(self,error,description,origin="",details=""):
    ServiceFailure.__init__(self,error,description,origin,details)


class AuthenticationFailure(ServiceFailure):

  def __init__(self,error,description,origin="",details=""):
    ServiceFailure.__init__(self,error,description,origin,details)


# Define classes for the standard scalar value types which are not
# supported natively to Python. An Opaque type is also defined to
# cater for value types when no class has been specifically defined
# for a value type.
#
# Note that the class for DateTime always expects a date/time value
# string to be in the format "YYYY-MM-DDThh:mm:ss". This is classified
# as the extended date/time format in ISO 8601 and is also the format
# used in the XML Schema Data Types specification. Similarly, the
# Date and Duration classes also adhere the XML Schema Data Types
# specification. Where the transport being used dictates some other
# format, a conversion will done at the point a message is sent or
# received.

class Boolean:

  def __init__(self,value=0):
    self.value = operator.truth(value)

  def __cmp__(self,other):
    if isinstance(other,Boolean):
      other = other.value
    return cmp(self.value,other)

  def __repr__(self):
    if self.value:
      return "True"
    else:
      return "False"

  def __cmp__(self,other):
    if isinstance(other,Boolean):
      other = other.value
    return cmp(self.value,other)

  def __int__(self):
    return self.value

  def __nonzero__(self):
    return self.value

False = Boolean(0)
True = Boolean(1)


class Binary:

  def __init__(self,data=None):
    self.data = data

  def __cmp__(self,other):
    if isinstance(other,Binary):
      other = other.data
    return cmp(self.data,other)

  def __str__(self):
    return self.data

  def __repr__(self):
    return "Binary(%s)" % `self.data`


class Date:

  def __init__(self,value=None):
    if value == None:
      value = time.time()
    t = type(value)
    if t is not types.StringType:
      if t is not types.TupleType:
	value = time.localtime(value)
      value = time.strftime("%Y-%m-%d",value)
    self.value = value

  def __cmp__(self,other):
    if isinstance(other,Date):
      other = other.value
    return cmp(self.value,other)

  def __str__(self):
    return str(self.value)

  def __repr__(self):
    return "Date(%s)" % `self.value`


class DateTime:

  def __init__(self,value=None):
    if value == None:
      value = time.time()
    t = type(value)
    if t is not types.StringType:
      if t is not types.TupleType:
	value = time.localtime(value)
      value = time.strftime("%Y-%m-%dT%H:%M:%S",value)
    self.value = value

  def __cmp__(self,other):
    if isinstance(other,DateTime):
      other = other.value
    return cmp(self.value,other)

  def __str__(self):
    return str(self.value)

  def __repr__(self):
    return "DateTime(%s)" % `self.value`


class Time:

  def __init__(self,value="00:00:00"):
    self.value = value

  def __cmp__(self,other):
    if isinstance(other,Time):
      other = other.value
    return cmp(self.value,other)

  def __str__(self):
    return str(self.value)

  def __repr__(self):
    return "Time(%s)" % `self.value`


class Duration:

  def __init__(self,value="P0DT0H0M0S"):
    self.value = value

  def __cmp__(self,other):
    if isinstance(other,Duration):
      other = other.value
    return cmp(self.value,other)

  def __str__(self):
    return str(self.value)

  def __repr__(self):
    return "Duration(%s)" % `self.value`


class Opaque:

  def __init__(self,type,data):
      self.type = type
      self.data = data

  def __cmp__(self,other):
    if isinstance(other,Opaque):
      other = (other.type,other.data)
    return cmp((self.type,self.data),other)

  def __repr__(self):
    return "Opaque(%s,%s)" % (`self.type`,`self.data`)


# Define routines for encoding and decoding of scalar value types.

_encoders = {}
_decoders = {}

def encoder(pythonType,encoder):
  if type(pythonType).__name__ == "class":
    valueType = pythonType.__name__
    moduleName = pythonType.__module__
  else:
    valueType = pythonType.__name__
    moduleName = ""
  _encoders[(moduleName,valueType)] = encoder

def decoder(valueType,decoder):
  _decoders[valueType] = decoder

def encodeObject(object):
  if type(object).__name__ == "instance":
    pythonType = object.__class__
    typeName = pythonType.__name__
    moduleName = pythonType.__module__
  else:
    typeName = type(object).__name__
    moduleName = ""
  lookupKey = (moduleName,typeName)
  if _encoders.has_key(lookupKey):
    callback = _encoders[lookupKey]
    return callback(object)
  else:
    raise TypeError("no encoder for type " + str(lookupKey))

def decodeValue(valueType,string):
  if _decoders.has_key(valueType):
    callback = _decoders[valueType]
    return callback(valueType,string)
  else:
    return Opaque(valueType,string)


# Define decoder/encoder routines for scalar builtin types.

def _encode_None(object):
  return ("","")

def _decode_None(name,object):
  return None

encoder(types.NoneType,_encode_None)
decoder("",_decode_None)

def _encode_Int(object):
  return ("xsd:int",str(object))

def _decode_Int(name,string):
  try:
    return int(string)
  except ValueError:
    return long(string)

encoder(types.IntType,_encode_Int)
decoder("xsd:byte",_decode_Int)
decoder("xsd:short",_decode_Int)
decoder("xsd:int",_decode_Int)
decoder("xsd:long",_decode_Int)
decoder("xsd:unsignedByte",_decode_Int)
decoder("xsd:unsignedShort",_decode_Int)
decoder("xsd:unsignedInt",_decode_Int)
decoder("xsd:unsignedLong",_decode_Int)
decoder("xsd:integer",_decode_Int)

def _encode_Long(object):
  return ("xsd:long",str(object))

encoder(types.LongType,_encode_Long)

def _encode_Float(object):
  return ("xsd:double",str(object))

def _decode_Float(name,string):
  return float(string)

encoder(types.FloatType,_encode_Float)
decoder("xsd:float",_decode_Float)
decoder("xsd:double",_decode_Float)
decoder("xsd:real",_decode_Float)

def _encode_String(object):
  return ("xsd:string",object)

def _decode_String(name,string):
  return string

encoder(types.StringType,_encode_String)
decoder("xsd:string",_decode_String)


# Define decoder/encoder routines for scalar extended types.

def _encode_Boolean(object):
  return ("xsd:boolean",str(operator.truth(object.value)))

def _decode_Boolean(name,string):
  if string in ["false","0"]:
    return False
  elif string in ["true","1"]:
    return True
  raise TypeError("invalid encoding for boolean type")

encoder(Boolean,_encode_Boolean)
decoder("xsd:boolean",_decode_Boolean)

def _encode_Binary(object):
  return ("xsd:base64Binary",object.data)

def _decode_Binary(name,string):
  return Binary(string)

encoder(Binary,_encode_Binary)
decoder("xsd:base64Binary",_decode_Binary)

def _encode_Date(object):
  return ("xsd:date",object.value)

def _decode_Date(name,string):
  return Date(string)

encoder(Date,_encode_Date)
decoder("xsd:date",_decode_Date)

def _encode_DateTime(object):
  return ("xsd:dateTime",object.value)

def _decode_DateTime(name,string):
  return DateTime(string)

encoder(DateTime,_encode_DateTime)
decoder("xsd:dateTime",_decode_DateTime)

def _encode_Time(object):
  return ("xsd:time",object.value)

def _decode_Time(name,string):
  return Time(string)

encoder(Time,_encode_Time)
decoder("xsd:time",_decode_Time)

def _encode_Duration(object):
  return ("xsd:duration",object.value)

def _decode_Duration(name,string):
  return Duration(string)

encoder(Duration,_encode_Duration)
decoder("xsd:duration",_decode_Duration)

def _encode_Opaque(object):
  return (object.type,object.data)

encoder(Opaque,_encode_Opaque)


# Define decoder/encoder routines for some other scalar types specified
# in Python which which aren't in the basic set of supported scalar types.

def _encode_Complex(object):
  return ("python:complex",repr(object))

def _decode_Complex(name,string):
  object = eval(string,{},{})
  if type(object) != types.ComplexType:
    raise TypeError("invalid encoding for complex type")
  return object

encoder(types.ComplexType,_encode_Complex)
decoder("python:complex",_decode_Complex)


# Determine what the best XML parser available is.

class _XmlSlowParser(xmllib.XMLParser):

  def __init__(self,target):
    self.unknown_starttag = target.start
    self.handle_data = target.data
    self.unknown_endtag = target.end
    xmllib.XMLParser.__init__(self)


class _XmlFastParser:

  def __init__(self, target):
    self.finish_starttag = target.start
    self.finish_endtag = target.end
    self.handle_data = target.data
    self.handle_xml = target.xml
    self.parser = sgmlop.XMLParser()
    self.parser.register(self)
    self.feed = self.parser.feed
    self.entity = { "amp": "&", "gt": ">", "lt": "<",
	"apos": "'", "quot": '"' }

  def close(self):
    try:
      self.parser.close()
    finally:
      self.parser = self.feed = None

  def handle_proc(self,tag,attr):
    m = re.search("encoding\s*=\s*['\"]([^\"']+)[\"']", attr)
    if m:
      self.handle_xml(m.group(1),1)

  def handle_entityref(self,entity):
    try:
      self.handle_data(self.entity[entity])
    except KeyError:
      self.handle_data("&%s;"% entity)


if sgmlop:
  _XmlParser = _XmlFastParser
else:
  _XmlParser = _XmlSlowParser


# Define decoder/encoder helper classes for the X-RPC protocol.

class _Encoder:

  def __init__(self):
    self._objects = {}
    self._body = []

  def convert(self,object):
    self._objects = {}
    self._body = []
    self._encodeObject(object)
    result = string.join(self._body,"")
    self._objects = {}
    self._body = []
    return result

  def _track(self,object):
    if object:
      i = id(object)
      if self._objects.has_key(i):
	raise TypeError("recursive data structure")
      self._objects[i] = None

  def _append(self,string):
    self._body.append(string)

  def _encodeObject(self,object,name=None):
    if type(object) == types.TupleType:
      self._encodeArray(object,name)
    elif type(object) == types.ListType:
      self._encodeArray(object,name)
    elif type(object) == types.DictionaryType:
      self._encodeIndex(object,name)
    else:
      self._encodeScalar(object,name)

  def _encodeArray(self,array,name=None):
    self._track(array)
    self._append("<array")
    if name != None:
      name = cgi.escape(name,1)
      self._append(" name=\"%s\">\n" % name)
    else:
      self._append(">\n")
    for object in array:
      self._encodeObject(object)
    self._append("</array>\n")

  def _encodeIndex(self,index,name=None):
    self._track(index)
    self._append("<index")
    if name != None:
      name = cgi.escape(name,1)
      self._append(" name=\"%s\">\n" % name)
    else:
      self._append(">\n")
    for key in index.keys():
      if type(key) != types.StringType:
	raise RuntimeError("dictionary keys must be strings")
      self._encodeObject(index[key],key)
    self._append("</index>\n")

  def _encodeScalar(self,scalar,name=None):
    type,string = encodeObject(scalar)
    if type == "xsd:base64Binary":
      string = base64.encodestring(string)
    else:
      string = cgi.escape(string)
    self._append("<scalar")
    type = cgi.escape(type,1)
    if type != "":
      if name != None:
	name = cgi.escape(name,1)
	self._append(" name=\"%s\" type=\"%s\">" % (name,type))
      else:
	self._append(" type=\"%s\">" % type)
    else:
      if name != None:
	name = cgi.escape(name,1)
	self._append(" name=\"%s\">" % name)
      else:
	self._append(">")
    self._append(string)
    self._append("</scalar>\n")


class _Decoder:

  def __init__(self):
    self._level = 0
    self._data = []
    self._root = []
    self._last = self._root
    self._stack = [self._root]
    self._type = None
    self._name = None
    self._parser = _XmlParser(self)

  def _fail(self):
    raise RuntimeError("badly formatted message")

  def feed(self,data):
    self._parser.feed(data)

  def result(self):
    self._parser.close()
    self._parser = None
    if len(self._root) != 1:
      self._fail()
    return self._root[0]

  def _push(self,object,name=None):
    if name != None:
      if type(self._last) != types.DictType:
	self._fail()
      else:
	self._last[name] = object
    else:
      if type(self._last) != types.ListType:
	self._fail()
      else:
	self._last.append(object)
    self._last = object
    self._stack.append(object)

  def _pop(self):
    del self._stack[-1]
    self._last = self._stack[-1]

  def start(self,tag,attrs):
    name = None
    if attrs.has_key("name"):
      name = attrs["name"]
    if tag == "scalar":
      if attrs.has_key("type"):
	self._type = attrs["type"]
      self._name = name
    elif tag == "array":
      self._push([],name)
    elif tag == "index":
      self._push({},name)
    self._data = []
    self._level = self._level + 1

  def data(self,content):
    self._data.append(content)

  def end(self,tag):
    self._level = self._level - 1
    if tag == "scalar":
      if self._type == None:
        self._type = ""
      data = string.join(self._data,"")
      if self._type == "xsd:base64Binary":
        data = base64.decodestring(data)
      scalar = decodeValue(self._type,data)
      self._push(scalar,self._name)
      self._pop()
      self._type = None
      self._data = []
    elif tag == "array":
      self._pop()
    elif tag == "index":
      self._pop()
    else:
      self._fail()


class _EndPoint:

  _debug = 0
  _request = ""
  _response = ""

  def __init__(self,url,login=None,password=None):
    self._encoder = _Encoder()
    self._url = url
    self._login = login
    self._password = password
    self._type,url = urllib.splittype(url)
    self._host,self._path = urllib.splithost(url)
    if not self._path:
      self._path = "/"

  def call(self,method,params):
    try:
      if _EndPoint._debug:
	_EndPoint._request = ""
	_EndPoint._response = ""
      if self._type != "http":
	error = CLIENT_SERVER_ADDRESS_INVALID
	description = CLIENT_SERVER_ADDRESS_INVALID_MESSAGE
	origin = "netrpc"
	details = self._url
	raise AddressInvalid(error,description,origin,details)

      try:
	body = self._encoder.convert(params)
      except ServiceFailure:
        raise
      except:
	error = CLIENT_REQUEST_ENCODING_ERROR
	description = CLIENT_REQUEST_ENCODING_ERROR_MESSAGE
	origin = "netrpc"
	details = exceptionDescription()
	raise ServiceFailure(error,description,origin,details)

      try:
	request = httplib.HTTP(self._host)
	request.putrequest("POST",self._path)
	request.putheader("Host",self._host)
	request.putheader("Connection","Close")
	request.putheader("Content-Type","application/x-rpc")
	request.putheader("Content-Length",str(len(body)))
	request.putheader("User-Agent","OSE/X-RPC)")
	if self._login != None and self._password != None:
	  info = "%s:%s" % (self._login,self._password)
	  info = base64.encodestring(info)
	  info = "Basic %s" % string.strip(info)
	  request.putheader("Authorization",info)
	request.putheader("X-RPC-Version","1.1")
	request.putheader("X-RPC-Content-Type","request")
	request.putheader("X-RPC-Request-Method",method)
	request.endheaders()
	if _EndPoint._debug:
	  headers = "POST %s HTTP/1.0" % self._path
	  headers = "%s\nHost: %s" % (headers,self._host)
	  headers = "%s\nConnection: Close" % headers
	  headers = "%s\nContent-Type: application/x-rpc" % headers
	  headers = "%s\nContent-Length: %s" % (headers,str(len(body)))
	  headers = "%s\nUser-Agent: OSE/X-RPC)" % headers
	  headers = "%s\nX-RPC-Version: 1.1" % headers
	  headers = "%s\nX-RPC-Content-Type: request" % headers
	  headers = "%s\nX-RPC-Request-Method: %s" % (headers,method)
	  _EndPoint._request = headers + "\n\n" + body
	request.send(body)
      except socket.error,exception:
	error = CLIENT_SERVER_UNAVAILABLE
	description = CLIENT_SERVER_UNAVAILABLE_MESSAGE
	origin = "netrpc"
	details = exceptionDescription()
	raise ServiceUnavailable(error,description,origin,details)

      error,description,headers = request.getreply()
      if _EndPoint._debug:
	_EndPoint._response = "HTTP/1.0 %d %s\r\n" % (error,description)
        for line in headers.headers:
	  _EndPoint._response = _EndPoint._response + line;
	_EndPoint._response = string.replace(_EndPoint._response,"\r\n","\n")
      if error != 200:
	origin = "netrpc"
	response = request.getfile()
	details = response.read()
	if _EndPoint._debug:
	  _EndPoint._response = _EndPoint._response + "\n" + details
	response.close()
	if error == 401:
	  raise AuthenticationFailure(error,description,origin,details)
	elif error == 404:
	  raise ServiceUnavailable(error,description,origin,details)
	else:
	  raise TransportFailure(error,description,origin,details)

      error = CLIENT_RESPONSE_HEADERS_INVALID
      description = CLIENT_RESPONSE_HEADERS_INVALID_MESSAGE
      origin = "netrpc"
      if not headers.has_key("content-type"):
	details = "The Content-Type header does not exist."
	raise ServiceFailure(error,description,origin,details)
      if headers["content-type"] != "application/x-rpc":
	details = "Unexpected value of %s for Content-Type."
	details = details % `headers["content-type"]`
	raise ServiceFailure(error,description,origin,details)
      if not headers.has_key("content-length"):
	details = "The Content-Length header does not exist."
	raise ServiceFailure(error,description,origin,details)
      try:
	expect = int(headers["content-length"])
      except:
	details = "Content length not an integer, but %s."
	details = details % `headers["content-length"]`
	raise ServiceFailure(error,description,origin,details)
      if not headers.has_key("x-rpc-content-type"):
	details = "The X-RPC-Content-Type header does not exist."
	raise ServiceFailure(error,description,origin,details)
      if headers["x-rpc-content-type"] not in ["response","failure"]:
	details = "Unexpected value of %s for X-RPC-Content-Type."
	details = details % `headers["x-rpc-content-type"]`
	raise ServiceFailure(error,description,origin,details)
      if not headers.has_key("x-rpc-version"):
	details = "The X-RPC-Version header does not exist."
	raise ServiceFailure(error,description,origin,details)
      if headers["x-rpc-version"] not in ["1.1"]:
	details = "Unexpected value of %s for X-RPC-Version."
	details = details % `headers["x-rpc-version"]`
	raise ServiceFailure(error,description,origin,details)

      try:
	response = request.getfile()
	decoder = _Decoder()
	actual = 0
	if _EndPoint._debug:
	  _EndPoint._response = _EndPoint._response + "\n"
	while 1:
	  content = response.read(4096)
	  if not content:
	    break
          actual = actual + len(content)
	  if _EndPoint._debug:
	    _EndPoint._response = _EndPoint._response + content
	  decoder.feed(content)
	if expect != actual:
	  error = CLIENT_RESPONSE_HEADERS_INVALID
	  description = CLIENT_RESPONSE_HEADERS_INVALID_MESSAGE
	  origin = "netrpc"
	  details = "Length of content was %d, expected %d."
	  details = details % (actual,expect)
	  raise ServiceFailure(error,description,origin,details)
	result = decoder.result()
	response.close()
      except ServiceFailure:
        raise
      except:
	error = CLIENT_RESPONSE_DECODING_ERROR
	description = CLIENT_RESPONSE_DECODING_ERROR_MESSAGE
	origin = "netrpc"
	details = exceptionDescription()
	raise ServiceFailure(error,description,origin,details)

      if headers["x-rpc-content-type"] == "failure":
	try:
	  error = result["error"]
	  description = result["description"]
	  origin = result["origin"]
	  details = result["details"]
	except:
	  error = CLIENT_RESPONSE_DECODING_ERROR
	  description = CLIENT_RESPONSE_DECODING_ERROR_MESSAGE
	  origin = "netrpc"
	  details = exceptionDescription()
	  raise ServiceFailure(error,description,origin,details)
	raise ServiceFailure(error,description,origin,details)
      else:
	return result

    except ServiceFailure:
      raise

    except:
      error = CLIENT_UNKNOWN_ERROR
      description = CLIENT_UNKNOWN_ERROR_MESSAGE
      origin = "netrpc"
      details = exceptionDetails()
      raise ServiceFailure(error,description,origin,details)


# Define remote service handle and magic method dispatcher.

class _Method:

  def __init__(self,endpoint,method):
    self._endpoint = endpoint
    self._method = method

  def __call__(self,*args):
    return self._endpoint.call(self._method,args)

  def __getattr__(self,method):
    return _Method(self._endpoint,"%s.%s" % (self._method,method))


class RemoteService:

  def __init__(self,url,login=None,password=None):
    self._endpoint = _EndPoint(url,login,password)

  def __getattr__(self,method):
    return _Method(self._endpoint,method)
