# This makefile is only suitable for Microsoft NMAKE and Microsoft C++.

# You must use the same options here as in MS.MK. You must have built
# the main OSE library using MS.MAK first as this makefile picks up the
# same objects rather than recompile them.

DEBUG = NO

THREADS = YES
EXCEPTIONS = YES
BOOL = YES
DLL = YES
STDLIB = NO

CC = CL
AR = LIB

LIBNAME = _ose

PYTHON_VERSION = 15
PYTHON_ROOT = c:/Progra~1/Python

TCL_VERSION = 80
TCL_ROOT = c:/Progra~1/Tcl

CPPFLAGS0 = \
 -DWIN32 \
 -DEXPAND_TEMPLATES \
 -I../config/X86_WIN32 \
 -I../include \
 -I$(PYTHON_ROOT)/include \
 -I$(TCL_ROOT)/include

!IF "$(DEBUG)" == "YES"
CFLAGS0 = -Zi -Od
!ELSE
CFLAGS0 = -O
!ENDIF

CFLAGS1 = -GX


!IF "$(THREADS)" == "YES"

CPPFLAGS2 = -DHAVE_WIN32_THREADS

!IF "$(DEBUG)" == "YES"
CFLAGS2 = -MDd
!ELSE
CFLAGS2 = -MD
!ENDIF

!ELSE

DLL = NO

!IF "$(DEBUG)" == "YES"
CFLAGS2 = -MLd
!ELSE
CFLAGS2 = -ML
!ENDIF

!ENDIF

!IF "$(EXCEPTIONS)" == "YES"

CPPFLAGS3 = -DHAVE_EXCEPTIONS

!ENDIF

!IF "$(BOOL)" == "YES"

CPPFLAGS4 = -DHAVE_BOOL

!ENDIF

!IF "$(THREADS)" == "YES"
!IF "$(DLL)" == "YES"
CPPFLAGS5 = -DPIC -DDLL_EXPORT_OSE
!ENDIF
!ENDIF

!IF "$(STDLIB)" == "YES"

CPPFLAGS6 = -DENV_CXXSTDLIB

!ENDIF

OBJECTS = \
 otctclqueue.obj \
 otcpyagent.obj \
 otcpydsptch.obj \
 otcpyechosl.obj \
 otcpyexchnc.obj \
 otcpyexchng.obj \
 otcpyfailre.obj \
 otcpyfilesl.obj \
 otcpyfilesr.obj \
 otcpyhttpdm.obj \
 otcpyhttpsl.obj \
 otcpyhttpsr.obj \
 otcpyjob.obj \
 otcpylogger.obj \
 otcpymnitor.obj \
 otcpymodule.obj \
 otcpypbntfn.obj \
 otcpyqueue.obj \
 otcpyreport.obj \
 otcpyrpcgwy.obj \
 otcpyrpcslt.obj \
 otcpyrquest.obj \
 otcpyrspnse.obj \
 otcpysbscrp.obj \
 otcpyservce.obj \
 otcpysrvbnd.obj \
 otcpyloader.obj

CPPFLAGS = $(CPPFLAGS0) $(CPPFLAGS1) $(CPPFLAGS2) \
 $(CPPFLAGS3) $(CPPFLAGS4) $(CPPFLAGS5) $(CPPFLAGS6)

CFLAGS = $(CFLAGS0) $(CFLAGS1) $(CFLAGS2) $(CFLAGS3)

!IF "$(CPPFLAGS5)" != ""
LIB : $(LIBNAME).dll
!ELSE
LIB : $(LIBNAME).lib
!ENDIF

$(LIBNAME).lib : $(OBJECTS)
	$(AR) -nologo -out:$@ *.obj

$(LIBNAME).dll : $(OBJECTS)
	$(CC) /LD -o $@ *.obj $(LDLIBS1) \
         $(PYTHON_ROOT)/libs/python$(PYTHON_VERSION).lib \
         $(TCL_ROOT)/lib/tk$(TCL_VERSION).lib \
         $(TCL_ROOT)/lib/tcl$(TCL_VERSION).lib \
         WSOCK32.LIB USER32.LIB

otctclqueue.obj : ../lib/OTC-TCL/tcl/otctclqueue.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-TCL/tcl/otctclqueue.cc

otcpyagent.obj : ../lib/OTC-PY/python/otcpyagent.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyagent.cc

otcpydsptch.obj : ../lib/OTC-PY/python/otcpydsptch.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpydsptch.cc

otcpyechosl.obj : ../lib/OTC-PY/python/otcpyechosl.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyechosl.cc

otcpyexchnc.obj : ../lib/OTC-PY/python/otcpyexchnc.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyexchnc.cc

otcpyexchng.obj : ../lib/OTC-PY/python/otcpyexchng.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyexchng.cc

otcpyfailre.obj : ../lib/OTC-PY/python/otcpyfailre.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyfailre.cc

otcpyfilesl.obj : ../lib/OTC-PY/python/otcpyfilesl.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyfilesl.cc

otcpyfilesr.obj : ../lib/OTC-PY/python/otcpyfilesr.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyfilesr.cc

otcpyhttpdm.obj : ../lib/OTC-PY/python/otcpyhttpdm.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyhttpdm.cc

otcpyhttpsl.obj : ../lib/OTC-PY/python/otcpyhttpsl.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyhttpsl.cc

otcpyhttpsr.obj : ../lib/OTC-PY/python/otcpyhttpsr.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyhttpsr.cc

otcpyjob.obj : ../lib/OTC-PY/python/otcpyjob.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyjob.cc

otcpylogger.obj : ../lib/OTC-PY/python/otcpylogger.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpylogger.cc

otcpymnitor.obj : ../lib/OTC-PY/python/otcpymnitor.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpymnitor.cc

otcpymodule.obj : ../lib/OTC-PY/python/otcpymodule.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpymodule.cc

otcpypbntfn.obj : ../lib/OTC-PY/python/otcpypbntfn.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpypbntfn.cc

otcpyqueue.obj : ../lib/OTC-PY/python/otcpyqueue.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyqueue.cc

otcpyreport.obj : ../lib/OTC-PY/python/otcpyreport.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyreport.cc

otcpyrpcgwy.obj : ../lib/OTC-PY/python/otcpyrpcgwy.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyrpcgwy.cc

otcpyrpcslt.obj : ../lib/OTC-PY/python/otcpyrpcslt.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyrpcslt.cc

otcpyrquest.obj : ../lib/OTC-PY/python/otcpyrquest.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyrquest.cc

otcpyrspnse.obj : ../lib/OTC-PY/python/otcpyrspnse.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyrspnse.cc

otcpysbscrp.obj : ../lib/OTC-PY/python/otcpysbscrp.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpysbscrp.cc

otcpyservce.obj : ../lib/OTC-PY/python/otcpyservce.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpyservce.cc

otcpysrvbnd.obj : ../lib/OTC-PY/python/otcpysrvbnd.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../lib/OTC-PY/python/otcpysrvbnd.cc

otcpyloader.obj : ../python/otcpyloader.cc
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -Tp ../python/otcpyloader.cc

